@@ -7,9 +7,8 @@ from ...config.injection_context import InjectionContext
 from ...connections.models.connection_record import ConnectionRecord
 from ...core.protocol_registry import ProtocolRegistry
 from ...messaging.agent_message import AgentMessage, AgentMessageSchema
-from ...messaging.error import MessageParseError
+from ...messaging.util import datetime_now
 
-# FIXME: We shouldn't rely on a hardcoded message version here.
 from ...protocols.problem_report.v1_0.message import ProblemReport
 
 from ...transport.inbound.message import InboundMessage
@@ -22,15 +21,15 @@ from .. import dispatcher as test_module
 def make_context() -> InjectionContext:
     context = InjectionContext()
     context.injector.bind_instance(ProtocolRegistry, ProtocolRegistry())
+
+    collector = test_module.Collector()
+    context.injector.bind_instance(test_module.Collector, collector)
+
     return context
 
 
 def make_inbound(payload) -> InboundMessage:
-    return InboundMessage(payload, MessageReceipt())
-
-
-# def make_connection_record() -> ConnectionRecord:
-#   return ConnectionRecord()
+    return InboundMessage(payload, MessageReceipt(thread_id="dummy-thread"))
 
 
 class Receiver:
@@ -95,7 +94,14 @@ class TestDispatcher(AsyncTestCase):
 
         with async_mock.patch.object(
             StubAgentMessageHandler, "handle", autospec=True
-        ) as handler_mock:
+        ) as handler_mock, async_mock.patch.object(
+            test_module, "ConnectionManager", autospec=True
+        ) as conn_mgr_mock:
+            conn_mgr_mock.return_value = async_mock.MagicMock(
+                find_inbound_connection=async_mock.CoroutineMock(
+                    return_value=async_mock.MagicMock(connection_id="dummy")
+                )
+            )
             await dispatcher.queue_message(make_inbound(message), rcv.send)
             await dispatcher.task_queue
             handler_mock.assert_awaited_once()
@@ -133,6 +139,67 @@ class TestDispatcher(AsyncTestCase):
                 handler_mock.call_args[0][2], test_module.DispatcherResponder
             )
 
+    async def test_dispatch_versioned_message_no_message_class(self):
+        context = make_context()
+        context.enforce_typing = False
+        registry = await context.inject(ProtocolRegistry)
+        registry.register_message_types(
+            {StubAgentMessage.Meta.message_type: StubAgentMessage},
+            version_definition={
+                "major_version": 1,
+                "minimum_minor_version": 0,
+                "current_minor_version": 1,
+                "path": "v1_1",
+            },
+        )
+        dispatcher = test_module.Dispatcher(context)
+        await dispatcher.setup()
+        rcv = Receiver()
+        message = {"@type": "proto-name/1.1/no-such-message-type"}
+
+        with async_mock.patch.object(
+            StubAgentMessageHandler, "handle", autospec=True
+        ) as handler_mock:
+            await dispatcher.queue_message(make_inbound(message), rcv.send)
+            await dispatcher.task_queue
+            assert rcv.messages and isinstance(rcv.messages[0][1], OutboundMessage)
+            payload = json.loads(rcv.messages[0][1].payload)
+            assert payload["@type"] == ProblemReport.Meta.message_type
+
+    async def test_dispatch_versioned_message_message_class_deserialize_x(self):
+        context = make_context()
+        context.enforce_typing = False
+        registry = await context.inject(ProtocolRegistry)
+        registry.register_message_types(
+            {StubAgentMessage.Meta.message_type: StubAgentMessage},
+            version_definition={
+                "major_version": 1,
+                "minimum_minor_version": 0,
+                "current_minor_version": 1,
+                "path": "v1_1",
+            },
+        )
+        dispatcher = test_module.Dispatcher(context)
+        await dispatcher.setup()
+        rcv = Receiver()
+        message = {"@type": "proto-name/1.1/no-such-message-type"}
+
+        with async_mock.patch.object(
+            StubAgentMessageHandler, "handle", autospec=True
+        ) as handler_mock, async_mock.patch.object(
+            registry, "resolve_message_class", async_mock.MagicMock()
+        ) as mock_resolve:
+            mock_resolve.return_value = async_mock.MagicMock(
+                deserialize=async_mock.MagicMock(
+                    side_effect=test_module.BaseModelError()
+                )
+            )
+            await dispatcher.queue_message(make_inbound(message), rcv.send)
+            await dispatcher.task_queue
+            assert rcv.messages and isinstance(rcv.messages[0][1], OutboundMessage)
+            payload = json.loads(rcv.messages[0][1].payload)
+            assert payload["@type"] == ProblemReport.Meta.message_type
+
     async def test_dispatch_versioned_message_handle_greater_succeeds(self):
         context = make_context()
         context.enforce_typing = False
@@ -199,3 +266,38 @@ class TestDispatcher(AsyncTestCase):
         assert rcv.messages and isinstance(rcv.messages[0][1], OutboundMessage)
         payload = json.loads(rcv.messages[0][1].payload)
         assert payload["@type"] == ProblemReport.Meta.message_type
+
+    async def test_dispatch_log(self):
+        context = make_context()
+        context.enforce_typing = False
+        registry = await context.inject(ProtocolRegistry)
+        registry.register_message_types(
+            {StubAgentMessage.Meta.message_type: StubAgentMessage}
+        )
+
+        dispatcher = test_module.Dispatcher(context)
+        await dispatcher.setup()
+
+        mock_task = async_mock.MagicMock(
+            exc_info=(KeyError, KeyError("sample exception"), "..."),
+            ident="abc",
+            timing={
+                "queued": 1234567890,
+                "unqueued": 1234567899,
+                "started": 1234567901,
+                "ended": 1234567999,
+            },
+        )
+        dispatcher.log_task(mock_task)
+
+    async def test_create_outbound_send_webhook(self):
+        context = make_context()
+        context.message_receipt = async_mock.MagicMock(in_time=datetime_now())
+        context.settings = {"timing.enabled": True}
+        message = StubAgentMessage()
+        responder = test_module.DispatcherResponder(
+            context, message, None, async_mock.CoroutineMock()
+        )
+        result = await responder.create_outbound(message)
+        assert json.loads(result.payload)["@type"] == StubAgentMessage.Meta.message_type
+        await responder.send_webhook("topic", "payload")
