@@ -3,14 +3,26 @@
 import json
 
 from aiohttp import web
-from aiohttp_apispec import docs, response_schema
+from aiohttp_apispec import docs, match_info_schema, querystring_schema, response_schema
 from marshmallow import fields, Schema
 
-from .base import BaseHolder
-from ..messaging.valid import INDY_CRED_DEF_ID, INDY_REV_REG_ID, INDY_SCHEMA_ID
+from .base import BaseHolder, HolderError
+from ..messaging.valid import (
+    INDY_CRED_DEF_ID,
+    INDY_REV_REG_ID,
+    INDY_SCHEMA_ID,
+    INDY_WQL,
+    NATURAL_NUM,
+    WHOLE_NUM,
+    UUIDFour,
+)
 from ..wallet.error import WalletNotFoundError
 
 
+class AttributeMimeTypesResultSchema(Schema):
+    """Result schema for credential attribute MIME type."""
+
+
 class RawEncCredAttrSchema(Schema):
     """Credential attribute schema."""
 
@@ -60,13 +72,32 @@ class CredentialSchema(Schema):
     witness = fields.Nested(WitnessSchema)
 
 
-class CredentialListSchema(Schema):
+class CredentialsListSchema(Schema):
     """Result schema for a credential query."""
 
     results = fields.List(fields.Nested(CredentialSchema()))
 
 
+class CredentialsListQueryStringSchema(Schema):
+    """Parameters and validators for query string in credentials list query."""
+
+    start = fields.Int(description="Start index", required=False, **WHOLE_NUM,)
+    count = fields.Int(
+        description="Maximum number to retrieve", required=False, **NATURAL_NUM,
+    )
+    wql = fields.Str(description="(JSON) WQL query", required=False, **INDY_WQL,)
+
+
+class CredIdMatchInfoSchema(Schema):
+    """Path parameters and validators for request taking credential id."""
+
+    credential_id = fields.Str(
+        description="Credential identifier", required=True, example=UUIDFour.EXAMPLE
+    )
+
+
 @docs(tags=["credentials"], summary="Fetch a credential from wallet by id")
+@match_info_schema(CredIdMatchInfoSchema())
 @response_schema(CredentialSchema(), 200)
 async def credentials_get(request: web.BaseRequest):
     """
@@ -81,19 +112,41 @@ async def credentials_get(request: web.BaseRequest):
     """
     context = request.app["request_context"]
 
-    credential_id = request.match_info["id"]
+    credential_id = request.match_info["credential_id"]
 
     holder: BaseHolder = await context.inject(BaseHolder)
     try:
         credential = await holder.get_credential(credential_id)
-    except WalletNotFoundError:
-        raise web.HTTPNotFound()
+    except WalletNotFoundError as err:
+        raise web.HTTPNotFound(reason=err.roll_up) from err
 
     credential_json = json.loads(credential)
     return web.json_response(credential_json)
 
 
+@docs(tags=["credentials"], summary="Get attribute MIME types from wallet")
+@match_info_schema(CredIdMatchInfoSchema())
+@response_schema(AttributeMimeTypesResultSchema(), 200)
+async def credentials_attr_mime_types_get(request: web.BaseRequest):
+    """
+    Request handler for getting credential attribute MIME types.
+
+    Args:
+        request: aiohttp request object
+
+    Returns:
+        The MIME types response
+
+    """
+    context = request.app["request_context"]
+    credential_id = request.match_info["credential_id"]
+    holder: BaseHolder = await context.inject(BaseHolder)
+
+    return web.json_response(await holder.get_mime_type(credential_id))
+
+
 @docs(tags=["credentials"], summary="Remove a credential from the wallet by id")
+@match_info_schema(CredIdMatchInfoSchema())
 async def credentials_remove(request: web.BaseRequest):
     """
     Request handler for searching connection records.
@@ -107,37 +160,22 @@ async def credentials_remove(request: web.BaseRequest):
     """
     context = request.app["request_context"]
 
-    credential_id = request.match_info["id"]
+    credential_id = request.match_info["credential_id"]
 
     holder: BaseHolder = await context.inject(BaseHolder)
     try:
         await holder.delete_credential(credential_id)
-    except WalletNotFoundError:
-        raise web.HTTPNotFound()
+    except WalletNotFoundError as err:
+        raise web.HTTPNotFound(reason=err.roll_up) from err
 
     return web.json_response({})
 
 
 @docs(
-    tags=["credentials"],
-    parameters=[
-        {
-            "name": "start",
-            "in": "query",
-            "schema": {"type": "string"},
-            "required": False,
-        },
-        {
-            "name": "count",
-            "in": "query",
-            "schema": {"type": "string"},
-            "required": False,
-        },
-        {"name": "wql", "in": "query", "schema": {"type": "string"}, "required": False},
-    ],
-    summary="Fetch credentials from wallet",
+    tags=["credentials"], summary="Fetch credentials from wallet",
 )
-@response_schema(CredentialListSchema(), 200)
+@querystring_schema(CredentialsListQueryStringSchema())
+@response_schema(CredentialsListSchema(), 200)
 async def credentials_list(request: web.BaseRequest):
     """
     Request handler for searching credential records.
@@ -163,7 +201,10 @@ async def credentials_list(request: web.BaseRequest):
     count = int(count) if isinstance(count, str) else 10
 
     holder: BaseHolder = await context.inject(BaseHolder)
-    credentials = await holder.get_credentials(start, count, wql)
+    try:
+        credentials = await holder.get_credentials(start, count, wql)
+    except HolderError as err:
+        raise web.HTTPBadRequest(reason=err.roll_up) from err
 
     return web.json_response({"results": credentials})
 
@@ -173,8 +214,31 @@ async def register(app: web.Application):
 
     app.add_routes(
         [
-            web.get("/credential/{id}", credentials_get),
-            web.post("/credential/{id}/remove", credentials_remove),
-            web.get("/credentials", credentials_list),
+            web.get("/credential/{credential_id}", credentials_get, allow_head=False),
+            web.get(
+                "/credential/mime-types/{credential_id}",
+                credentials_attr_mime_types_get,
+                allow_head=False,
+            ),
+            web.post("/credential/{credential_id}/remove", credentials_remove),
+            web.get("/credentials", credentials_list, allow_head=False),
         ]
     )
+
+
+def post_process_routes(app: web.Application):
+    """Amend swagger API."""
+
+    # Add top-level tags description
+    if "tags" not in app._state["swagger_dict"]:
+        app._state["swagger_dict"]["tags"] = []
+    app._state["swagger_dict"]["tags"].append(
+        {
+            "name": "credentials",
+            "description": "Holder credential management",
+            "externalDocs": {
+                "description": "Overview",
+                "url": ("https://w3c.github.io/vc-data-model/#credentials"),
+            },
+        }
+    )
