@@ -1,6 +1,6 @@
 """Action menu perform request message handler."""
 
-from aries_cloudagent.messaging.base_handler import (
+from .....messaging.base_handler import (
     BaseHandler,
     BaseResponder,
     RequestContext,
