@@ -4,7 +4,7 @@ from typing import Mapping
 
 from marshmallow import fields
 
-from aries_cloudagent.messaging.agent_message import AgentMessage, AgentMessageSchema
+from .....messaging.agent_message import AgentMessage, AgentMessageSchema
 
 from ..message_types import PERFORM, PROTOCOL_PACKAGE
 
