@@ -74,14 +74,36 @@ class TestConnectionRoutes(AsyncTestCase):
                     }  # sorted
                 )
 
+    async def test_connections_list_x(self):
+        context = RequestContext(base_context=InjectionContext(enforce_typing=False))
+        mock_req = async_mock.MagicMock()
+        mock_req.app = {
+            "request_context": context,
+        }
+        mock_req.query = {
+            "invitation_id": "dummy",
+            "initiator": ConnectionRecord.INITIATOR_SELF,
+        }
+
+        with async_mock.patch.object(
+            test_module, "ConnectionRecord", autospec=True
+        ) as mock_conn_rec:
+            mock_conn_rec.STATE_INVITATION = ConnectionRecord.STATE_INVITATION
+            mock_conn_rec.STATE_INACTIVE = ConnectionRecord.STATE_INACTIVE
+            mock_conn_rec.query = async_mock.CoroutineMock(
+                side_effect=test_module.StorageError()
+            )
+
+            with self.assertRaises(test_module.web.HTTPBadRequest):
+                await test_module.connections_list(mock_req)
+
     async def test_connections_retrieve(self):
         context = RequestContext(base_context=InjectionContext(enforce_typing=False))
         mock_req = async_mock.MagicMock()
         mock_req.app = {
             "request_context": context,
         }
-        mock_req.match_info = {"id": "dummy"}
-
+        mock_req.match_info = {"conn_id": "dummy"}
         mock_conn_rec = async_mock.MagicMock()
         mock_conn_rec.serialize = async_mock.MagicMock(return_value={"hello": "world"})
 
@@ -90,7 +112,6 @@ class TestConnectionRoutes(AsyncTestCase):
         ) as mock_conn_rec_retrieve_by_id, async_mock.patch.object(
             test_module.web, "json_response"
         ) as mock_response:
-
             mock_conn_rec_retrieve_by_id.return_value = mock_conn_rec
 
             await test_module.connections_retrieve(mock_req)
@@ -102,7 +123,7 @@ class TestConnectionRoutes(AsyncTestCase):
         mock_req.app = {
             "request_context": context,
         }
-        mock_req.match_info = {"id": "dummy"}
+        mock_req.match_info = {"conn_id": "dummy"}
 
         with async_mock.patch.object(
             test_module.ConnectionRecord, "retrieve_by_id", async_mock.CoroutineMock()
@@ -112,6 +133,26 @@ class TestConnectionRoutes(AsyncTestCase):
             with self.assertRaises(test_module.web.HTTPNotFound):
                 await test_module.connections_retrieve(mock_req)
 
+    async def test_connections_retrieve_x(self):
+        context = RequestContext(base_context=InjectionContext(enforce_typing=False))
+        mock_req = async_mock.MagicMock()
+        mock_req.app = {
+            "request_context": context,
+        }
+        mock_req.match_info = {"conn_id": "dummy"}
+        mock_conn_rec = async_mock.MagicMock()
+        mock_conn_rec.serialize = async_mock.MagicMock(
+            side_effect=test_module.BaseModelError()
+        )
+
+        with async_mock.patch.object(
+            test_module.ConnectionRecord, "retrieve_by_id", async_mock.CoroutineMock()
+        ) as mock_conn_rec_retrieve_by_id:
+            mock_conn_rec_retrieve_by_id.return_value = mock_conn_rec
+
+            with self.assertRaises(test_module.web.HTTPBadRequest):
+                await test_module.connections_retrieve(mock_req)
+
     async def test_connections_create_invitation(self):
         context = RequestContext(base_context=InjectionContext(enforce_typing=False))
         mock_req = async_mock.MagicMock()
@@ -120,10 +161,10 @@ class TestConnectionRoutes(AsyncTestCase):
             "request_context": context,
         }
         mock_req.query = {
-            "accept": "auto",
+            "auto_accept": "true",
             "alias": "alias",
-            "public": 1,
-            "multi_use": 1,
+            "public": "true",
+            "multi_use": "true",
         }
 
         with async_mock.patch.object(
@@ -154,6 +195,30 @@ class TestConnectionRoutes(AsyncTestCase):
                 }
             )
 
+    async def test_connections_create_invitation_x(self):
+        context = RequestContext(base_context=InjectionContext(enforce_typing=False))
+        mock_req = async_mock.MagicMock()
+        context.update_settings({"public_invites": True})
+        mock_req.app = {
+            "request_context": context,
+        }
+        mock_req.query = {
+            "auto_accept": "true",
+            "alias": "alias",
+            "public": "true",
+            "multi_use": "true",
+        }
+
+        with async_mock.patch.object(
+            test_module, "ConnectionManager", autospec=True
+        ) as mock_conn_mgr:
+            mock_conn_mgr.return_value.create_invitation = async_mock.CoroutineMock(
+                side_effect=test_module.ConnectionManagerError()
+            )
+
+            with self.assertRaises(test_module.web.HTTPBadRequest):
+                await test_module.connections_create_invitation(mock_req)
+
     async def test_connections_create_invitation_public_forbidden(self):
         context = RequestContext(base_context=InjectionContext(enforce_typing=False))
         mock_req = async_mock.MagicMock()
@@ -162,10 +227,10 @@ class TestConnectionRoutes(AsyncTestCase):
             "request_context": context,
         }
         mock_req.query = {
-            "accept": "auto",
+            "auto_accept": "true",
             "alias": "alias",
-            "public": 1,
-            "multi_use": 1,
+            "public": "true",
+            "multi_use": "true",
         }
 
         with self.assertRaises(test_module.web.HTTPForbidden):
@@ -179,7 +244,7 @@ class TestConnectionRoutes(AsyncTestCase):
         }
         mock_req.json = async_mock.CoroutineMock()
         mock_req.query = {
-            "accept": "auto",
+            "auto_accept": "true",
             "alias": "alias",
         }
 
@@ -200,6 +265,31 @@ class TestConnectionRoutes(AsyncTestCase):
             await test_module.connections_receive_invitation(mock_req)
             mock_response.assert_called_once_with(mock_conn_rec.serialize.return_value)
 
+    async def test_connections_receive_invitation_bad(self):
+        context = RequestContext(base_context=InjectionContext(enforce_typing=False))
+        mock_req = async_mock.MagicMock()
+        mock_req.app = {
+            "request_context": context,
+        }
+        mock_req.json = async_mock.CoroutineMock()
+        mock_req.query = {
+            "auto_accept": "true",
+            "alias": "alias",
+        }
+
+        mock_conn_rec = async_mock.MagicMock()
+        mock_conn_rec.serialize = async_mock.MagicMock()
+
+        with async_mock.patch.object(
+            test_module.ConnectionInvitation, "deserialize", autospec=True
+        ) as mock_inv_deser, async_mock.patch.object(
+            test_module, "ConnectionManager", autospec=True
+        ) as mock_conn_mgr:
+            mock_inv_deser.side_effect = test_module.BaseModelError()
+
+            with self.assertRaises(test_module.web.HTTPBadRequest):
+                await test_module.connections_receive_invitation(mock_req)
+
     async def test_connections_receive_invitation_forbidden(self):
         context = RequestContext(base_context=InjectionContext(enforce_typing=False))
         context.update_settings({"admin.no_receive_invites": True})
@@ -215,7 +305,7 @@ class TestConnectionRoutes(AsyncTestCase):
             "request_context": context,
             "outbound_message_router": async_mock.CoroutineMock(),
         }
-        mock_req.match_info = {"id": "dummy"}
+        mock_req.match_info = {"conn_id": "dummy"}
         mock_req.query = {
             "my_label": "label",
             "my_endpoint": "http://endpoint.ca",
@@ -245,7 +335,7 @@ class TestConnectionRoutes(AsyncTestCase):
             "request_context": context,
             "outbound_message_router": async_mock.CoroutineMock(),
         }
-        mock_req.match_info = {"id": "dummy"}
+        mock_req.match_info = {"conn_id": "dummy"}
 
         with async_mock.patch.object(
             test_module.ConnectionRecord, "retrieve_by_id", async_mock.CoroutineMock()
@@ -255,6 +345,27 @@ class TestConnectionRoutes(AsyncTestCase):
             with self.assertRaises(test_module.web.HTTPNotFound):
                 await test_module.connections_accept_invitation(mock_req)
 
+    async def test_connections_accept_invitation_x(self):
+        context = RequestContext(base_context=InjectionContext(enforce_typing=False))
+        mock_req = async_mock.MagicMock()
+        mock_req.app = {
+            "request_context": context,
+            "outbound_message_router": async_mock.CoroutineMock(),
+        }
+        mock_req.match_info = {"conn_id": "dummy"}
+
+        with async_mock.patch.object(
+            test_module.ConnectionRecord, "retrieve_by_id", async_mock.CoroutineMock()
+        ) as mock_conn_rec_retrieve_by_id, async_mock.patch.object(
+            test_module, "ConnectionManager", autospec=True
+        ) as mock_conn_mgr:
+            mock_conn_mgr.return_value.create_request = async_mock.CoroutineMock(
+                side_effect=test_module.ConnectionManagerError()
+            )
+
+            with self.assertRaises(test_module.web.HTTPBadRequest):
+                await test_module.connections_accept_invitation(mock_req)
+
     async def test_connections_accept_request(self):
         context = RequestContext(base_context=InjectionContext(enforce_typing=False))
         mock_req = async_mock.MagicMock()
@@ -262,7 +373,7 @@ class TestConnectionRoutes(AsyncTestCase):
             "request_context": context,
             "outbound_message_router": async_mock.CoroutineMock(),
         }
-        mock_req.match_info = {"id": "dummy"}
+        mock_req.match_info = {"conn_id": "dummy"}
         mock_req.query = {
             "my_endpoint": "http://endpoint.ca",
         }
@@ -290,7 +401,7 @@ class TestConnectionRoutes(AsyncTestCase):
             "request_context": context,
             "outbound_message_router": async_mock.CoroutineMock(),
         }
-        mock_req.match_info = {"id": "dummy"}
+        mock_req.match_info = {"conn_id": "dummy"}
 
         with async_mock.patch.object(
             test_module.ConnectionRecord, "retrieve_by_id", async_mock.CoroutineMock()
@@ -300,6 +411,29 @@ class TestConnectionRoutes(AsyncTestCase):
             with self.assertRaises(test_module.web.HTTPNotFound):
                 await test_module.connections_accept_request(mock_req)
 
+    async def test_connections_accept_request_x(self):
+        context = RequestContext(base_context=InjectionContext(enforce_typing=False))
+        mock_req = async_mock.MagicMock()
+        mock_req.app = {
+            "request_context": context,
+            "outbound_message_router": async_mock.CoroutineMock(),
+        }
+        mock_req.match_info = {"conn_id": "dummy"}
+
+        with async_mock.patch.object(
+            test_module.ConnectionRecord, "retrieve_by_id", async_mock.CoroutineMock()
+        ) as mock_conn_rec_retrieve_by_id, async_mock.patch.object(
+            test_module, "ConnectionManager", autospec=True
+        ) as mock_conn_mgr, async_mock.patch.object(
+            test_module.web, "json_response"
+        ) as mock_response:
+            mock_conn_mgr.return_value.create_response = async_mock.CoroutineMock(
+                side_effect=test_module.ConnectionManagerError()
+            )
+
+            with self.assertRaises(test_module.web.HTTPBadRequest):
+                await test_module.connections_accept_request(mock_req)
+
     async def test_connections_establish_inbound(self):
         context = RequestContext(base_context=InjectionContext(enforce_typing=False))
         mock_req = async_mock.MagicMock()
@@ -307,11 +441,10 @@ class TestConnectionRoutes(AsyncTestCase):
             "request_context": context,
             "outbound_message_router": async_mock.CoroutineMock(),
         }
-        mock_req.match_info = {"id": "dummy", "ref_id": "ref"}
+        mock_req.match_info = {"conn_id": "dummy", "ref_id": "ref"}
         mock_req.query = {
             "my_endpoint": "http://endpoint.ca",
         }
-
         mock_conn_rec = async_mock.MagicMock()
 
         with async_mock.patch.object(
@@ -321,7 +454,6 @@ class TestConnectionRoutes(AsyncTestCase):
         ) as mock_conn_mgr, async_mock.patch.object(
             test_module.web, "json_response"
         ) as mock_response:
-
             mock_conn_rec_retrieve_by_id.return_value = mock_conn_rec
             mock_conn_mgr.return_value.establish_inbound = async_mock.CoroutineMock()
 
@@ -335,7 +467,7 @@ class TestConnectionRoutes(AsyncTestCase):
             "request_context": context,
             "outbound_message_router": async_mock.CoroutineMock(),
         }
-        mock_req.match_info = {"id": "dummy", "ref_id": "ref"}
+        mock_req.match_info = {"conn_id": "dummy", "ref_id": "ref"}
 
         with async_mock.patch.object(
             test_module.ConnectionRecord, "retrieve_by_id", async_mock.CoroutineMock()
@@ -345,14 +477,38 @@ class TestConnectionRoutes(AsyncTestCase):
             with self.assertRaises(test_module.web.HTTPNotFound):
                 await test_module.connections_establish_inbound(mock_req)
 
+    async def test_connections_establish_inbound_x(self):
+        context = RequestContext(base_context=InjectionContext(enforce_typing=False))
+        mock_req = async_mock.MagicMock()
+        mock_req.app = {
+            "request_context": context,
+            "outbound_message_router": async_mock.CoroutineMock(),
+        }
+        mock_req.match_info = {"conn_id": "dummy", "ref_id": "ref"}
+        mock_req.query = {
+            "my_endpoint": "http://endpoint.ca",
+        }
+        mock_conn_rec = async_mock.MagicMock()
+
+        with async_mock.patch.object(
+            test_module.ConnectionRecord, "retrieve_by_id", async_mock.CoroutineMock()
+        ) as mock_conn_rec_retrieve_by_id, async_mock.patch.object(
+            test_module, "ConnectionManager", autospec=True
+        ) as mock_conn_mgr:
+            mock_conn_rec_retrieve_by_id.return_value = mock_conn_rec
+            mock_conn_mgr.return_value.establish_inbound = async_mock.CoroutineMock(
+                side_effect=test_module.ConnectionManagerError()
+            )
+            with self.assertRaises(test_module.web.HTTPBadRequest):
+                await test_module.connections_establish_inbound(mock_req)
+
     async def test_connections_remove(self):
         context = RequestContext(base_context=InjectionContext(enforce_typing=False))
         mock_req = async_mock.MagicMock()
         mock_req.app = {
             "request_context": context,
         }
-        mock_req.match_info = {"id": "dummy"}
-
+        mock_req.match_info = {"conn_id": "dummy"}
         mock_conn_rec = async_mock.MagicMock()
         mock_conn_rec.delete_record = async_mock.CoroutineMock()
 
@@ -372,7 +528,7 @@ class TestConnectionRoutes(AsyncTestCase):
         mock_req.app = {
             "request_context": context,
         }
-        mock_req.match_info = {"id": "dummy"}
+        mock_req.match_info = {"conn_id": "dummy"}
 
         mock_conn_rec = async_mock.MagicMock()
 
@@ -384,6 +540,27 @@ class TestConnectionRoutes(AsyncTestCase):
             with self.assertRaises(test_module.web.HTTPNotFound):
                 await test_module.connections_remove(mock_req)
 
+    async def test_connections_remove_x(self):
+        context = RequestContext(base_context=InjectionContext(enforce_typing=False))
+        mock_req = async_mock.MagicMock()
+        mock_req.app = {
+            "request_context": context,
+        }
+        mock_req.match_info = {"conn_id": "dummy"}
+        mock_conn_rec = async_mock.MagicMock(
+            delete_record=async_mock.CoroutineMock(
+                side_effect=test_module.StorageError()
+            )
+        )
+
+        with async_mock.patch.object(
+            test_module.ConnectionRecord, "retrieve_by_id", async_mock.CoroutineMock()
+        ) as mock_conn_rec_retrieve_by_id:
+            mock_conn_rec_retrieve_by_id.return_value = mock_conn_rec
+
+            with self.assertRaises(test_module.web.HTTPBadRequest):
+                await test_module.connections_remove(mock_req)
+
     async def test_connections_create_static(self):
         context = RequestContext(base_context=InjectionContext(enforce_typing=False))
         mock_req = async_mock.MagicMock()
@@ -403,10 +580,10 @@ class TestConnectionRoutes(AsyncTestCase):
             }
         )
         mock_req.query = {
-            "accept": "auto",
+            "auto_accept": "true",
             "alias": "alias",
         }
-        mock_req.match_info = {"id": "dummy"}
+        mock_req.match_info = {"conn_id": "dummy"}
 
         mock_conn_rec = async_mock.MagicMock()
         mock_conn_rec.serialize = async_mock.MagicMock()
@@ -422,7 +599,6 @@ class TestConnectionRoutes(AsyncTestCase):
         ) as mock_conn_mgr, async_mock.patch.object(
             test_module.web, "json_response"
         ) as mock_response:
-
             mock_conn_mgr.return_value.create_static_connection = async_mock.CoroutineMock(
                 return_value=(mock_my_info, mock_their_info, mock_conn_rec)
             )
@@ -439,9 +615,57 @@ class TestConnectionRoutes(AsyncTestCase):
                 }
             )
 
+    async def test_connections_create_static_x(self):
+        context = RequestContext(base_context=InjectionContext(enforce_typing=False))
+        mock_req = async_mock.MagicMock()
+        mock_req.app = {
+            "request_context": context,
+        }
+        mock_req.json = async_mock.CoroutineMock(
+            return_value={
+                "my_seed": "my_seed",
+                "my_did": "my_did",
+                "their_seed": "their_seed",
+                "their_did": "their_did",
+                "their_verkey": "their_verkey",
+                "their_endpoint": "their_endpoint",
+                "their_role": "their_role",
+                "alias": "alias",
+            }
+        )
+        mock_req.query = {
+            "auto_accept": "true",
+            "alias": "alias",
+        }
+        mock_req.match_info = {"conn_id": "dummy"}
+
+        mock_conn_rec = async_mock.MagicMock()
+        mock_conn_rec.serialize = async_mock.MagicMock()
+        mock_my_info = async_mock.MagicMock()
+        mock_my_info.did = "my_did"
+        mock_my_info.verkey = "my_verkey"
+        mock_their_info = async_mock.MagicMock()
+        mock_their_info.did = "their_did"
+        mock_their_info.verkey = "their_verkey"
+
+        with async_mock.patch.object(
+            test_module, "ConnectionManager", autospec=True
+        ) as mock_conn_mgr:
+            mock_conn_mgr.return_value.create_static_connection = async_mock.CoroutineMock(
+                side_effect=test_module.WalletError()
+            )
+
+            with self.assertRaises(test_module.web.HTTPBadRequest):
+                await test_module.connections_create_static(mock_req)
+
     async def test_register(self):
         mock_app = async_mock.MagicMock()
         mock_app.add_routes = async_mock.MagicMock()
 
         await test_module.register(mock_app)
         mock_app.add_routes.assert_called_once()
+
+    async def test_post_process_routes(self):
+        mock_app = async_mock.MagicMock(_state={"swagger_dict": {}})
+        test_module.post_process_routes(mock_app)
+        assert "tags" in mock_app._state["swagger_dict"]
