@@ -1,11 +1,12 @@
 """Feature discovery admin routes."""
 
 from aiohttp import web
-from aiohttp_apispec import docs, response_schema
-
+from aiohttp_apispec import docs, querystring_schema, response_schema
 from marshmallow import fields, Schema
 
-from aries_cloudagent.core.protocol_registry import ProtocolRegistry
+from ....core.protocol_registry import ProtocolRegistry
+
+from .message_types import SPEC_URI
 
 
 class QueryResultSchema(Schema):
@@ -18,18 +19,16 @@ class QueryResultSchema(Schema):
     )
 
 
+class QueryFeaturesQueryStringSchema(Schema):
+    """Query string parameters for feature query."""
+
+    query = fields.Str(description="Query", required=False, example="did:sov:*")
+
+
 @docs(
-    tags=["server"],
-    summary="Query supported features",
-    parameters=[
-        {
-            "name": "query",
-            "in": "query",
-            "schema": {"type": "string"},
-            "required": False,
-        }
-    ],
+    tags=["server"], summary="Query supported features",
 )
+@querystring_schema(QueryFeaturesQueryStringSchema())
 @response_schema(QueryResultSchema(), 200)
 async def query_features(request: web.BaseRequest):
     """
@@ -52,4 +51,19 @@ async def query_features(request: web.BaseRequest):
 async def register(app: web.Application):
     """Register routes."""
 
-    app.add_routes([web.get("/features", query_features)])
+    app.add_routes([web.get("/features", query_features, allow_head=False)])
+
+
+def post_process_routes(app: web.Application):
+    """Amend swagger API."""
+
+    # Add top-level tags description
+    if "tags" not in app._state["swagger_dict"]:
+        app._state["swagger_dict"]["tags"] = []
+    app._state["swagger_dict"]["tags"].append(
+        {
+            "name": "server",
+            "description": "Feature discovery",
+            "externalDocs": {"description": "Specification", "url": SPEC_URI},
+        }
+    )
