@@ -1,6 +1,6 @@
 """Handler for incoming invitation messages."""
 
-from aries_cloudagent.messaging.base_handler import (
+from .....messaging.base_handler import (
     BaseHandler,
     BaseResponder,
     HandlerException,
