@@ -2,7 +2,7 @@
 
 import json
 import logging
-from typing import Mapping, Text, Sequence, Tuple
+from typing import Mapping, Sequence, Text, Tuple
 
 from .messages.credential_ack import CredentialAck
 from .messages.credential_issue import CredentialIssue
@@ -81,8 +81,7 @@ class CredentialManager:
 
         Args:
             connection_id: Connection to create offer for
-            credential_proposal: The credential proposal with preview on
-                attribute values to use if auto_issue is enabled
+            credential_proposal: The credential proposal with preview
             auto_remove: Flag to automatically remove the record on completion
 
         Returns:
@@ -101,10 +100,10 @@ class CredentialManager:
             trace=(credential_proposal._trace is not None),
         )
         (credential_exchange, credential_offer) = await self.create_offer(
-            credential_exchange_record=credential_exchange,
+            cred_ex_record=credential_exchange,
             comment="create automated credential exchange",
         )
-        return credential_exchange, credential_offer
+        return (credential_exchange, credential_offer)
 
     async def create_proposal(
         self,
@@ -158,7 +157,7 @@ class CredentialManager:
 
         if auto_remove is None:
             auto_remove = not self.context.settings.get("preserve_exchange_records")
-        credential_exchange_record = V10CredentialExchange(
+        cred_ex_record = V10CredentialExchange(
             connection_id=connection_id,
             thread_id=credential_proposal_message._thread_id,
             initiator=V10CredentialExchange.INITIATOR_SELF,
@@ -169,10 +168,8 @@ class CredentialManager:
             auto_remove=auto_remove,
             trace=trace,
         )
-        await credential_exchange_record.save(
-            self.context, reason="create credential proposal"
-        )
-        return credential_exchange_record
+        await cred_ex_record.save(self.context, reason="create credential proposal")
+        return cred_ex_record
 
     async def receive_proposal(self) -> V10CredentialExchange:
         """
@@ -182,12 +179,11 @@ class CredentialManager:
             The resulting credential exchange record, created
 
         """
-        # go to cred def via ledger to get authoritative schema id
         credential_proposal_message = self.context.message
         connection_id = self.context.connection_record.connection_id
 
         # at this point, cred def and schema still open to potential negotiation
-        credential_exchange_record = V10CredentialExchange(
+        cred_ex_record = V10CredentialExchange(
             connection_id=connection_id,
             thread_id=credential_proposal_message._thread_id,
             initiator=V10CredentialExchange.INITIATOR_EXTERNAL,
@@ -202,51 +198,58 @@ class CredentialManager:
             ),
             trace=(credential_proposal_message._trace is not None),
         )
-        await credential_exchange_record.save(
-            self.context, reason="receive credential proposal"
-        )
+        await cred_ex_record.save(self.context, reason="receive credential proposal")
 
-        return credential_exchange_record
+        return cred_ex_record
 
     async def create_offer(
-        self, credential_exchange_record: V10CredentialExchange, comment: str = None
+        self, cred_ex_record: V10CredentialExchange, comment: str = None
     ) -> Tuple[V10CredentialExchange, CredentialOffer]:
         """
         Create a credential offer, update credential exchange record.
 
         Args:
-            credential_exchange_record: Credential exchange to create offer for
+            cred_ex_record: Credential exchange to create offer for
             comment: optional human-readable comment to set in offer message
 
         Returns:
             A tuple (credential exchange record, credential offer message)
 
         """
-        if credential_exchange_record.credential_proposal_dict:
-            credential_proposal_message = CredentialProposal.deserialize(
-                credential_exchange_record.credential_proposal_dict
-            )
-            credential_proposal_message.assign_trace_decorator(
-                self.context.settings, credential_exchange_record.trace
-            )
-            cred_def_id = await self._match_sent_cred_def_id(
-                {
-                    t: getattr(credential_proposal_message, t)
-                    for t in CRED_DEF_TAGS
-                    if getattr(credential_proposal_message, t)
-                }
-            )
-
-            cred_preview = credential_proposal_message.credential_proposal
-        else:
-            cred_def_id = credential_exchange_record.credential_definition_id
-            cred_preview = None
 
         async def _create(cred_def_id):
             issuer: BaseIssuer = await self.context.inject(BaseIssuer)
             offer_json = await issuer.create_credential_offer(cred_def_id)
             return json.loads(offer_json)
 
+        credential_proposal_message = CredentialProposal.deserialize(
+            cred_ex_record.credential_proposal_dict
+        )
+        credential_proposal_message.assign_trace_decorator(
+            self.context.settings, cred_ex_record.trace
+        )
+        cred_def_id = await self._match_sent_cred_def_id(
+            {
+                t: getattr(credential_proposal_message, t)
+                for t in CRED_DEF_TAGS
+                if getattr(credential_proposal_message, t)
+            }
+        )
+        cred_preview = credential_proposal_message.credential_proposal
+
+        # vet attributes
+        ledger: BaseLedger = await self.context.inject(BaseLedger)
+        async with ledger:
+            schema_id = await ledger.credential_definition_id2schema_id(cred_def_id)
+            schema = await ledger.get_schema(schema_id)
+        schema_attrs = {attr for attr in schema["attrNames"]}
+        preview_attrs = {attr for attr in cred_preview.attr_dict()}
+        if preview_attrs != schema_attrs:
+            raise CredentialManagerError(
+                f"Preview attributes {preview_attrs} "
+                f"mismatch corresponding schema attributes {schema_attrs}"
+            )
+
         credential_offer = None
         cache_key = f"credential_offer::{cred_def_id}"
         cache: BaseCache = await self.context.inject(BaseCache, required=False)
@@ -266,25 +269,22 @@ class CredentialManager:
             offers_attach=[CredentialOffer.wrap_indy_offer(credential_offer)],
         )
 
-        credential_offer_message._thread = {
-            "thid": credential_exchange_record.thread_id
-        }
+        credential_offer_message._thread = {"thid": cred_ex_record.thread_id}
         credential_offer_message.assign_trace_decorator(
-            self.context.settings, credential_exchange_record.trace
+            self.context.settings, cred_ex_record.trace
         )
 
-        credential_exchange_record.thread_id = credential_offer_message._thread_id
-        credential_exchange_record.schema_id = credential_offer["schema_id"]
-        credential_exchange_record.credential_definition_id = credential_offer[
-            "cred_def_id"
-        ]
-        credential_exchange_record.state = V10CredentialExchange.STATE_OFFER_SENT
-        credential_exchange_record.credential_offer = credential_offer
-        await credential_exchange_record.save(
-            self.context, reason="create credential offer"
-        )
+        cred_ex_record.thread_id = credential_offer_message._thread_id
+        cred_ex_record.schema_id = credential_offer["schema_id"]
+        cred_ex_record.credential_definition_id = credential_offer["cred_def_id"]
+        cred_ex_record.state = V10CredentialExchange.STATE_OFFER_SENT
+        cred_ex_record.credential_offer = credential_offer
 
-        return (credential_exchange_record, credential_offer_message)
+        cred_ex_record.credential_offer_dict = credential_offer_message.serialize()
+
+        await cred_ex_record.save(self.context, reason="create credential offer")
+
+        return (cred_ex_record, credential_offer_message)
 
     async def receive_offer(self) -> V10CredentialExchange:
         """
@@ -302,29 +302,24 @@ class CredentialManager:
         schema_id = indy_offer["schema_id"]
         cred_def_id = indy_offer["cred_def_id"]
 
-        if credential_preview:
-            credential_proposal_dict = CredentialProposal(
-                comment=credential_offer_message.comment,
-                credential_proposal=credential_preview,
-                schema_id=schema_id,
-                cred_def_id=cred_def_id,
-            ).serialize()
-        else:
-            credential_proposal_dict = None
+        credential_proposal_dict = CredentialProposal(
+            comment=credential_offer_message.comment,
+            credential_proposal=credential_preview,
+            schema_id=schema_id,
+            cred_def_id=cred_def_id,
+        ).serialize()
 
         # Get credential exchange record (holder sent proposal first)
         # or create it (issuer sent offer first)
         try:
             (
-                credential_exchange_record
+                cred_ex_record
             ) = await V10CredentialExchange.retrieve_by_connection_and_thread(
                 self.context, connection_id, credential_offer_message._thread_id
             )
-            credential_exchange_record.credential_proposal_dict = (
-                credential_proposal_dict
-            )
+            cred_ex_record.credential_proposal_dict = credential_proposal_dict
         except StorageNotFoundError:  # issuer sent this offer free of any proposal
-            credential_exchange_record = V10CredentialExchange(
+            cred_ex_record = V10CredentialExchange(
                 connection_id=connection_id,
                 thread_id=credential_offer_message._thread_id,
                 initiator=V10CredentialExchange.INITIATOR_EXTERNAL,
@@ -333,25 +328,23 @@ class CredentialManager:
                 trace=(credential_offer_message._trace is not None),
             )
 
-        credential_exchange_record.credential_offer = indy_offer
-        credential_exchange_record.state = V10CredentialExchange.STATE_OFFER_RECEIVED
-        credential_exchange_record.schema_id = schema_id
-        credential_exchange_record.credential_definition_id = cred_def_id
+        cred_ex_record.credential_offer = indy_offer
+        cred_ex_record.state = V10CredentialExchange.STATE_OFFER_RECEIVED
+        cred_ex_record.schema_id = schema_id
+        cred_ex_record.credential_definition_id = cred_def_id
 
-        await credential_exchange_record.save(
-            self.context, reason="receive credential offer"
-        )
+        await cred_ex_record.save(self.context, reason="receive credential offer")
 
-        return credential_exchange_record
+        return cred_ex_record
 
     async def create_request(
-        self, credential_exchange_record: V10CredentialExchange, holder_did: str
+        self, cred_ex_record: V10CredentialExchange, holder_did: str
     ) -> Tuple[V10CredentialExchange, CredentialRequest]:
         """
         Create a credential request.
 
         Args:
-            credential_exchange_record: Credential exchange record
+            cred_ex_record: Credential exchange record
                 for which to create request
             holder_did: holder DID
 
@@ -359,8 +352,15 @@ class CredentialManager:
             A tuple (credential exchange record, credential request message)
 
         """
-        credential_definition_id = credential_exchange_record.credential_definition_id
-        credential_offer = credential_exchange_record.credential_offer
+        if cred_ex_record.state != V10CredentialExchange.STATE_OFFER_RECEIVED:
+            raise CredentialManagerError(
+                f"Credential exchange {cred_ex_record.credential_exchange_id} "
+                f"in {cred_ex_record.state} state "
+                f"(must be {V10CredentialExchange.STATE_OFFER_RECEIVED})"
+            )
+
+        credential_definition_id = cred_ex_record.credential_definition_id
+        credential_offer = cred_ex_record.credential_offer
 
         async def _create():
             ledger: BaseLedger = await self.context.inject(BaseLedger)
@@ -378,10 +378,10 @@ class CredentialManager:
                 "metadata": json.loads(metadata_json),
             }
 
-        if credential_exchange_record.credential_request:
+        if cred_ex_record.credential_request:
             self._logger.warning(
                 "create_request called multiple times for v1.0 credential exchange: %s",
-                credential_exchange_record.credential_exchange_id,
+                cred_ex_record.credential_exchange_id,
             )
         else:
             if "nonce" not in credential_offer:
@@ -403,30 +403,24 @@ class CredentialManager:
                 cred_req_result = await _create()
 
             (
-                credential_exchange_record.credential_request,
-                credential_exchange_record.credential_request_metadata,
+                cred_ex_record.credential_request,
+                cred_ex_record.credential_request_metadata,
             ) = (cred_req_result["request"], cred_req_result["metadata"])
 
         credential_request_message = CredentialRequest(
             requests_attach=[
-                CredentialRequest.wrap_indy_cred_req(
-                    credential_exchange_record.credential_request
-                )
+                CredentialRequest.wrap_indy_cred_req(cred_ex_record.credential_request)
             ]
         )
-        credential_request_message._thread = {
-            "thid": credential_exchange_record.thread_id
-        }
+        credential_request_message._thread = {"thid": cred_ex_record.thread_id}
         credential_request_message.assign_trace_decorator(
-            self.context.settings, credential_exchange_record.trace
+            self.context.settings, cred_ex_record.trace
         )
 
-        credential_exchange_record.state = V10CredentialExchange.STATE_REQUEST_SENT
-        await credential_exchange_record.save(
-            self.context, reason="create credential request"
-        )
+        cred_ex_record.state = V10CredentialExchange.STATE_REQUEST_SENT
+        await cred_ex_record.save(self.context, reason="create credential request")
 
-        return credential_exchange_record, credential_request_message
+        return (cred_ex_record, credential_request_message)
 
     async def receive_request(self):
         """
@@ -442,98 +436,101 @@ class CredentialManager:
         credential_request_message = self.context.message
         assert len(credential_request_message.requests_attach or []) == 1
         credential_request = credential_request_message.indy_cred_req(0)
+        connection_id = (
+            self.context.connection_record
+            and self.context.connection_record.connection_id
+        )
 
         (
-            credential_exchange_record
+            cred_ex_record
         ) = await V10CredentialExchange.retrieve_by_connection_and_thread(
-            self.context,
-            self.context.connection_record.connection_id,
-            credential_request_message._thread_id,
-        )
-        credential_exchange_record.credential_request = credential_request
-        credential_exchange_record.state = V10CredentialExchange.STATE_REQUEST_RECEIVED
-        await credential_exchange_record.save(
-            self.context, reason="receive credential request"
+            self.context, connection_id, credential_request_message._thread_id
         )
+        cred_ex_record.credential_request = credential_request
+        cred_ex_record.state = V10CredentialExchange.STATE_REQUEST_RECEIVED
+        await cred_ex_record.save(self.context, reason="receive credential request")
 
-        return credential_exchange_record
+        return cred_ex_record
 
     async def issue_credential(
-        self,
-        credential_exchange_record: V10CredentialExchange,
-        *,
-        comment: str = None,
-        credential_values: dict,
+        self, cred_ex_record: V10CredentialExchange, *, comment: str = None,
     ) -> Tuple[V10CredentialExchange, CredentialIssue]:
         """
         Issue a credential.
 
         Args:
-            credential_exchange_record: The credential exchange record
+            cred_ex_record: The credential exchange record
                 for which to issue a credential
             comment: optional human-readable comment pertaining to credential issue
-            credential_values: dict of credential attribute {name: value} pairs
 
         Returns:
             Tuple: (Updated credential exchange record, credential message)
 
         """
-        schema_id = credential_exchange_record.schema_id
+        if cred_ex_record.state != V10CredentialExchange.STATE_REQUEST_RECEIVED:
+            raise CredentialManagerError(
+                f"Credential exchange {cred_ex_record.credential_exchange_id} "
+                f"in {cred_ex_record.state} state "
+                f"(must be {V10CredentialExchange.STATE_REQUEST_RECEIVED})"
+            )
+
+        schema_id = cred_ex_record.schema_id
         registry = None
 
-        if credential_exchange_record.credential:
+        if cred_ex_record.credential:
             self._logger.warning(
                 "issue_credential called multiple times for "
                 + "v1.0 credential exchange: %s",
-                credential_exchange_record.credential_exchange_id,
+                cred_ex_record.credential_exchange_id,
             )
         else:
-            credential_offer = credential_exchange_record.credential_offer
-            credential_request = credential_exchange_record.credential_request
+            credential_offer = cred_ex_record.credential_offer
+            credential_request = cred_ex_record.credential_request
 
             ledger: BaseLedger = await self.context.inject(BaseLedger)
             async with ledger:
                 schema = await ledger.get_schema(schema_id)
                 credential_definition = await ledger.get_credential_definition(
-                    credential_exchange_record.credential_definition_id
+                    cred_ex_record.credential_definition_id
                 )
 
             if credential_definition["value"].get("revocation"):
                 issuer_rev_regs = await IssuerRevRegRecord.query_by_cred_def_id(
                     self.context,
-                    credential_exchange_record.credential_definition_id,
+                    cred_ex_record.credential_definition_id,
                     state=IssuerRevRegRecord.STATE_ACTIVE,
                 )
                 if not issuer_rev_regs:
                     raise CredentialManagerError(
                         "Cred def id {} has no active revocation registry".format(
-                            credential_exchange_record.credential_definition_id
+                            cred_ex_record.credential_definition_id
                         )
                     )
 
                 registry = await issuer_rev_regs[0].get_registry()
-                credential_exchange_record.revoc_reg_id = issuer_rev_regs[
-                    0
-                ].revoc_reg_id
+                cred_ex_record.revoc_reg_id = issuer_rev_regs[0].revoc_reg_id
                 tails_path = registry.tails_local_path
             else:
                 tails_path = None
 
+            credential_values = CredentialProposal.deserialize(
+                cred_ex_record.credential_proposal_dict
+            ).credential_proposal.attr_dict(decode=False)
             issuer: BaseIssuer = await self.context.inject(BaseIssuer)
             try:
                 (
                     credential_json,
-                    credential_exchange_record.revocation_id,
+                    cred_ex_record.revocation_id,
                 ) = await issuer.create_credential(
                     schema,
                     credential_offer,
                     credential_request,
                     credential_values,
-                    credential_exchange_record.revoc_reg_id,
+                    cred_ex_record.revoc_reg_id,
                     tails_path,
                 )
                 if registry and registry.max_creds == int(
-                    credential_exchange_record.revocation_id  # monotonic "1"-based
+                    cred_ex_record.revocation_id  # monotonic "1"-based
                 ):
                     await issuer_rev_regs[0].mark_full(self.context)
 
@@ -541,25 +538,23 @@ class CredentialManager:
                 await issuer_rev_regs[0].mark_full(self.context)
                 raise
 
-            credential_exchange_record.credential = json.loads(credential_json)
+            cred_ex_record.credential = json.loads(credential_json)
 
-        credential_exchange_record.state = V10CredentialExchange.STATE_ISSUED
-        await credential_exchange_record.save(self.context, reason="issue credential")
+        cred_ex_record.state = V10CredentialExchange.STATE_ISSUED
+        await cred_ex_record.save(self.context, reason="issue credential")
 
         credential_message = CredentialIssue(
             comment=comment,
             credentials_attach=[
-                CredentialIssue.wrap_indy_credential(
-                    credential_exchange_record.credential
-                )
+                CredentialIssue.wrap_indy_credential(cred_ex_record.credential)
             ],
         )
-        credential_message._thread = {"thid": credential_exchange_record.thread_id}
+        credential_message._thread = {"thid": cred_ex_record.thread_id}
         credential_message.assign_trace_decorator(
-            self.context.settings, credential_exchange_record.trace
+            self.context.settings, cred_ex_record.trace
         )
 
-        return (credential_exchange_record, credential_message)
+        return (cred_ex_record, credential_message)
 
     async def receive_credential(self) -> V10CredentialExchange:
         """
@@ -576,31 +571,27 @@ class CredentialManager:
         raw_credential = credential_message.indy_credential(0)
 
         (
-            credential_exchange_record
+            cred_ex_record
         ) = await V10CredentialExchange.retrieve_by_connection_and_thread(
             self.context,
             self.context.connection_record.connection_id,
             credential_message._thread_id,
         )
 
-        credential_exchange_record.raw_credential = raw_credential
-        credential_exchange_record.state = (
-            V10CredentialExchange.STATE_CREDENTIAL_RECEIVED
-        )
+        cred_ex_record.raw_credential = raw_credential
+        cred_ex_record.state = V10CredentialExchange.STATE_CREDENTIAL_RECEIVED
 
-        await credential_exchange_record.save(self.context, reason="receive credential")
-        return credential_exchange_record
+        await cred_ex_record.save(self.context, reason="receive credential")
+        return cred_ex_record
 
     async def store_credential(
-        self,
-        credential_exchange_record: V10CredentialExchange,
-        credential_id: str = None,
+        self, cred_ex_record: V10CredentialExchange, credential_id: str = None
     ) -> Tuple[V10CredentialExchange, CredentialAck]:
         """
         Store a credential in holder wallet; send ack to issuer.
 
         Args:
-            credential_exchange_record: credential exchange record
+            cred_ex_record: credential exchange record
                 with credential to store and ack
             credential_id: optional credential identifier to override default on storage
 
@@ -608,7 +599,14 @@ class CredentialManager:
             Tuple: (Updated credential exchange record, credential ack message)
 
         """
-        raw_credential = credential_exchange_record.raw_credential
+        if cred_ex_record.state != (V10CredentialExchange.STATE_CREDENTIAL_RECEIVED):
+            raise CredentialManagerError(
+                f"Credential exchange {cred_ex_record.credential_exchange_id} "
+                f"in {cred_ex_record.state} state "
+                f"(must be {V10CredentialExchange.STATE_CREDENTIAL_RECEIVED})"
+            )
+
+        raw_credential = cred_ex_record.raw_credential
         revoc_reg_def = None
         ledger: BaseLedger = await self.context.inject(BaseLedger)
         async with ledger:
@@ -625,28 +623,23 @@ class CredentialManager:
 
         holder: BaseHolder = await self.context.inject(BaseHolder)
         if (
-            credential_exchange_record.credential_proposal_dict
-            and "credential_proposal"
-            in credential_exchange_record.credential_proposal_dict
+            cred_ex_record.credential_proposal_dict
+            and "credential_proposal" in cred_ex_record.credential_proposal_dict
         ):
             mime_types = CredentialPreview.deserialize(
-                credential_exchange_record.credential_proposal_dict[
-                    "credential_proposal"
-                ]
+                cred_ex_record.credential_proposal_dict["credential_proposal"]
             ).mime_types()
         else:
             mime_types = None
 
         if revoc_reg_def:
             revoc_reg = RevocationRegistry.from_definition(revoc_reg_def, True)
-            if not revoc_reg.has_local_tails_file(self.context):
-                await revoc_reg.retrieve_tails(self.context)
-
+            await revoc_reg.get_or_fetch_local_tails_path()
         try:
             credential_id = await holder.store_credential(
                 credential_definition,
                 raw_credential,
-                credential_exchange_record.credential_request_metadata,
+                cred_ex_record.credential_request_metadata,
                 mime_types,
                 credential_id=credential_id,
                 rev_reg_def=revoc_reg_def,
@@ -658,28 +651,27 @@ class CredentialManager:
         credential_json = await holder.get_credential(credential_id)
         credential = json.loads(credential_json)
 
-        credential_exchange_record.state = V10CredentialExchange.STATE_ACKED
-        credential_exchange_record.credential_id = credential_id
-        credential_exchange_record.credential = credential
-        credential_exchange_record.revoc_reg_id = credential.get("rev_reg_id", None)
-        credential_exchange_record.revocation_id = credential.get("cred_rev_id", None)
+        cred_ex_record.state = V10CredentialExchange.STATE_ACKED
+        cred_ex_record.credential_id = credential_id
+        cred_ex_record.credential = credential
+        cred_ex_record.revoc_reg_id = credential.get("rev_reg_id", None)
+        cred_ex_record.revocation_id = credential.get("cred_rev_id", None)
 
-        await credential_exchange_record.save(self.context, reason="store credential")
+        await cred_ex_record.save(self.context, reason="store credential")
 
         credential_ack_message = CredentialAck()
         credential_ack_message.assign_thread_id(
-            credential_exchange_record.thread_id,
-            credential_exchange_record.parent_thread_id,
+            cred_ex_record.thread_id, cred_ex_record.parent_thread_id
         )
         credential_ack_message.assign_trace_decorator(
-            self.context.settings, credential_exchange_record.trace
+            self.context.settings, cred_ex_record.trace
         )
 
-        if credential_exchange_record.auto_remove:
+        if cred_ex_record.auto_remove:
             # Delete the exchange record since we're done with it
-            await credential_exchange_record.delete_record(self.context)
+            await cred_ex_record.delete_record(self.context)
 
-        return (credential_exchange_record, credential_ack_message)
+        return (cred_ex_record, credential_ack_message)
 
     async def receive_credential_ack(self) -> V10CredentialExchange:
         """
@@ -691,21 +683,21 @@ class CredentialManager:
         """
         credential_ack_message = self.context.message
         (
-            credential_exchange_record
+            cred_ex_record
         ) = await V10CredentialExchange.retrieve_by_connection_and_thread(
             self.context,
             self.context.connection_record.connection_id,
             credential_ack_message._thread_id,
         )
 
-        credential_exchange_record.state = V10CredentialExchange.STATE_ACKED
-        await credential_exchange_record.save(self.context, reason="credential acked")
+        cred_ex_record.state = V10CredentialExchange.STATE_ACKED
+        await cred_ex_record.save(self.context, reason="credential acked")
 
-        if credential_exchange_record.auto_remove:
+        if cred_ex_record.auto_remove:
             # We're done with the exchange so delete
-            await credential_exchange_record.delete_record(self.context)
+            await cred_ex_record.delete_record(self.context)
 
-        return credential_exchange_record
+        return cred_ex_record
 
     async def revoke_credential(
         self, rev_reg_id: str, cred_rev_id: str, publish: bool = False
@@ -718,7 +710,8 @@ class CredentialManager:
         Args:
             rev_reg_id: revocation registry id
             cred_rev_id: credential revocation id
-            publish: whether to publish the resulting revocation registry delta
+            publish: whether to publish the resulting revocation registry delta,
+                along with any revocations pending against it
 
         """
         issuer: BaseIssuer = await self.context.inject(BaseIssuer)
@@ -731,42 +724,113 @@ class CredentialManager:
             )
 
         if publish:
-            # create entry and send to ledger
-            delta = json.loads(
-                await issuer.revoke_credentials(
-                    rev_reg_id, registry_record.tails_local_path, [cred_rev_id]
-                )
-            )
+            rev_reg = await revoc.get_ledger_registry(rev_reg_id)
+            await rev_reg.get_or_fetch_local_tails_path()
 
-            if delta:
-                registry_record.revoc_reg_entry = delta
+            # pick up pending revocations on input revocation registry
+            crids = list(set(registry_record.pending_pub + [cred_rev_id]))
+            (delta_json, _) = await issuer.revoke_credentials(
+                registry_record.revoc_reg_id, registry_record.tails_local_path, crids
+            )
+            if delta_json:
+                registry_record.revoc_reg_entry = json.loads(delta_json)
                 await registry_record.publish_registry_entry(self.context)
+                await registry_record.clear_pending(self.context)
+
         else:
             await registry_record.mark_pending(self.context, cred_rev_id)
 
-    async def publish_pending_revocations(self) -> Mapping[Text, Sequence[Text]]:
+    async def publish_pending_revocations(
+        self, rrid2crid: Mapping[Text, Sequence[Text]] = None
+    ) -> Mapping[Text, Sequence[Text]]:
         """
         Publish pending revocations to the ledger.
 
+        Args:
+            rrid2crid: Mapping from revocation registry identifiers to all credential
+                revocation identifiers within each to publish. Specify null/empty map
+                for all revocation registries. Specify empty sequence per revocation
+                registry identifier for all pending within the revocation registry;
+                e.g.,
+
+            ::
+
+                {} - publish all pending revocations from all revocation registries
+                {
+                    "R17v42T4pk...:4:R17v42T4pk...:3:CL:19:tag:CL_ACCUM:0": [],
+                    "R17v42T4pk...:4:R17v42T4pk...:3:CL:19:tag:CL_ACCUM:1": ["1", "2"]
+                } - publish:
+                    - all pending revocations from all revocation registry tagged 0
+                    - pending ["1", "2"] from revocation registry tagged 1
+                    - no pending revocations from any other revocation registries.
+
         Returns: mapping from each revocation registry id to its cred rev ids published.
         """
-
         result = {}
-
         issuer: BaseIssuer = await self.context.inject(BaseIssuer)
 
         registry_records = await IssuerRevRegRecord.query_by_pending(self.context)
         for registry_record in registry_records:
-            revoke_idxs = list(registry_record.pending_pub)
-            if revoke_idxs:
-                delta_json = await issuer.revoke_credentials(
+            rrid = registry_record.revoc_reg_id
+            crids = []
+            if not rrid2crid:
+                crids = registry_record.pending_pub
+            elif rrid in rrid2crid:
+                crids = [
+                    crid
+                    for crid in registry_record.pending_pub
+                    if crid in (rrid2crid[rrid] or []) or not rrid2crid[rrid]
+                ]
+            if crids:
+                (delta_json, failed_crids) = await issuer.revoke_credentials(
                     registry_record.revoc_reg_id,
                     registry_record.tails_local_path,
-                    revoke_idxs,
+                    crids,
                 )
                 registry_record.revoc_reg_entry = json.loads(delta_json)
                 await registry_record.publish_registry_entry(self.context)
-                result[registry_record.revoc_reg_id] = revoke_idxs
-                await registry_record.clear_pending(self.context)
+                published = [crid for crid in crids if crid not in failed_crids]
+                result[registry_record.revoc_reg_id] = published
+                await registry_record.clear_pending(self.context, published)
+
+        return result
+
+    async def clear_pending_revocations(
+        self, purge: Mapping[Text, Sequence[Text]] = None
+    ) -> Mapping[Text, Sequence[Text]]:
+        """
+        Clear pending revocation publications.
+
+        Args:
+            purge: Mapping from revocation registry identifiers to all credential
+                revocation identifiers within each to clear. Specify null/empty map
+                for all revocation registries. Specify empty sequence per revocation
+                registry identifier for all pending within the revocation registry;
+                e.g.,
+
+            ::
+
+                {} - clear all pending revocations from all revocation registries
+                {
+                    "R17v42T4pk...:4:R17v42T4pk...:3:CL:19:tag:CL_ACCUM:0": [],
+                    "R17v42T4pk...:4:R17v42T4pk...:3:CL:19:tag:CL_ACCUM:1": ["1", "2"]
+                } - clear:
+                    - all pending revocations from all revocation registry tagged 0
+                    - pending ["1", "2"] from revocation registry tagged 1
+                    - no pending revocations from any other revocation registries.
+
+        Returns:
+            mapping from revocation registry id to its remaining
+            cred rev ids still marked pending, omitting revocation registries
+            with no remaining pending publications.
+
+        """
+        result = {}
+        registry_records = await IssuerRevRegRecord.query_by_pending(self.context)
+        for registry_record in registry_records:
+            rrid = registry_record.revoc_reg_id
+            await registry_record.clear_pending(self.context, (purge or {}).get(rrid))
+            if registry_record.pending_pub:
+                result[rrid] = registry_record.pending_pub
 
         return result
