@@ -14,6 +14,7 @@ class TestRecord(UnitTestCase):
             state="state",
             presentation_proposal_dict={"prop": "dict"},
             presentation_request={"pres": "req"},
+            presentation_request_dict={"pres", "dict"},
             presentation={"pres": "indy"},
             verified=False,
             auto_present=True,
@@ -27,6 +28,7 @@ class TestRecord(UnitTestCase):
             "initiator": "init",
             "presentation_proposal_dict": {"prop": "dict"},
             "presentation_request": {"pres": "req"},
+            "presentation_request_dict": {"pres", "dict"},
             "presentation": {"pres": "indy"},
             "role": "role",
             "state": "state",
