@@ -3,25 +3,41 @@
 import json
 
 from aiohttp import web
-from aiohttp_apispec import docs, request_schema, response_schema
-from marshmallow import Schema, fields
+from aiohttp_apispec import (
+    docs,
+    match_info_schema,
+    querystring_schema,
+    request_schema,
+    response_schema,
+)
+from marshmallow import fields, Schema, validate, validates_schema
+from marshmallow.exceptions import ValidationError
 
 from ....connections.models.connection_record import ConnectionRecord
-from ....holder.base import BaseHolder
+from ....holder.base import BaseHolder, HolderError
+from ....ledger.error import LedgerError
 from ....messaging.decorators.attach_decorator import AttachDecorator
+from ....messaging.models.base import BaseModelError
 from ....messaging.valid import (
     INDY_CRED_DEF_ID,
     INDY_DID,
+    INDY_EXTRA_WQL,
     INDY_PREDICATE,
     INDY_SCHEMA_ID,
     INDY_VERSION,
     INT_EPOCH,
+    NATURAL_NUM,
     UUIDFour,
+    UUID4,
+    WHOLE_NUM,
 )
-from ....storage.error import StorageNotFoundError
+from ....storage.error import StorageError, StorageNotFoundError
 from ....indy.util import generate_pr_nonce
 
+from ...problem_report.v1_0 import internal_error
+
 from .manager import PresentationManager
+from .message_types import ATTACH_DECO_IDS, PRESENTATION_REQUEST, SPEC_URI
 from .messages.inner.presentation_preview import (
     PresentationPreview,
     PresentationPreviewSchema,
@@ -33,17 +49,53 @@ from .models.presentation_exchange import (
     V10PresentationExchangeSchema,
 )
 
-from .message_types import ATTACH_DECO_IDS, PRESENTATION_REQUEST
 
 from ....utils.tracing import trace_event, get_timer, AdminAPIMessageTracingSchema
 
 
+class V10PresentationExchangeListQueryStringSchema(Schema):
+    """Parameters and validators for presentation exchange list query."""
+
+    connection_id = fields.UUID(
+        description="Connection identifier",
+        required=False,
+        example=UUIDFour.EXAMPLE,  # typically but not necessarily a UUID4
+    )
+    thread_id = fields.UUID(
+        description="Thread identifier",
+        required=False,
+        example=UUIDFour.EXAMPLE,  # typically but not necessarily a UUID4
+    )
+    role = fields.Str(
+        description="Role assigned in presentation exchange",
+        required=False,
+        validate=validate.OneOf(
+            [
+                getattr(V10PresentationExchange, m)
+                for m in vars(V10PresentationExchange)
+                if m.startswith("ROLE_")
+            ]
+        ),
+    )
+    state = fields.Str(
+        description="Presentation exchange state",
+        required=False,
+        validate=validate.OneOf(
+            [
+                getattr(V10PresentationExchange, m)
+                for m in vars(V10PresentationExchange)
+                if m.startswith("STATE_")
+            ]
+        ),
+    )
+
+
 class V10PresentationExchangeListSchema(Schema):
-    """Result schema for an Aries#0037 v1.0 presentation exchange query."""
+    """Result schema for an Aries RFC 37 v1.0 presentation exchange query."""
 
     results = fields.List(
         fields.Nested(V10PresentationExchangeSchema()),
-        description="Aries#0037 v1.0 presentation exchange records",
+        description="Aries RFC 37 v1.0 presentation exchange records",
     )
 
 
@@ -65,16 +117,16 @@ class V10PresentationProposalRequestSchema(AdminAPIMessageTracingSchema):
         required=False,
         default=False,
     )
+    trace = fields.Bool(
+        description="Whether to trace event (default false)",
+        required=False,
+        example=False,
+    )
 
 
-class IndyProofReqSpecRestrictionsSchema(Schema):
+class IndyProofReqPredSpecRestrictionsSchema(Schema):
     """Schema for restrictions in attr or pred specifier indy proof request."""
 
-    credential_definition_id = fields.Str(
-        description="Credential definition identifier",
-        required=True,
-        **INDY_CRED_DEF_ID
-    )
     schema_id = fields.String(
         description="Schema identifier", required=False, **INDY_SCHEMA_ID
     )
@@ -93,37 +145,104 @@ class IndyProofReqSpecRestrictionsSchema(Schema):
     cred_def_id = fields.String(
         description="Credential definition identifier",
         required=False,
-        **INDY_CRED_DEF_ID
+        **INDY_CRED_DEF_ID,
     )
 
 
-class IndyProofReqNonRevoked(Schema):
+class IndyProofReqNonRevokedSchema(Schema):
     """Non-revocation times specification in indy proof request."""
 
-    from_epoch = fields.Int(
+    fro = fields.Int(
         description="Earliest epoch of interest for non-revocation proof",
-        required=True,
-        **INT_EPOCH
+        required=False,
+        data_key="from",
+        **INT_EPOCH,
     )
-    to_epoch = fields.Int(
+    to = fields.Int(
         description="Latest epoch of interest for non-revocation proof",
-        required=True,
-        **INT_EPOCH
+        required=False,
+        **INT_EPOCH,
     )
 
+    @validates_schema
+    def validate_fields(self, data, **kwargs):
+        """
+        Validate schema fields - must have from, to, or both.
+
+        Args:
+            data: The data to validate
+
+        Raises:
+            ValidationError: if data has neither from nor to
+
+        """
+        if not (data.get("from") or data.get("to")):
+            raise ValidationError(
+                "Non-revocation interval must have at least one end", ("fro", "to")
+            )
+
 
 class IndyProofReqAttrSpecSchema(Schema):
     """Schema for attribute specification in indy proof request."""
 
     name = fields.String(
-        example="favouriteDrink", description="Attribute name", required=True
+        example="favouriteDrink", description="Attribute name", required=False
     )
-    restrictions = fields.List(
-        fields.Nested(IndyProofReqSpecRestrictionsSchema()),
-        description="If present, credential must satisfy one of given restrictions",
+    names = fields.List(
+        fields.String(example="age"),
+        description="Attribute name group",
         required=False,
     )
-    non_revoked = fields.Nested(IndyProofReqNonRevoked(), required=False)
+    restrictions = fields.List(
+        fields.Dict(
+            keys=fields.Str(
+                validate=validate.Regexp(
+                    "^schema_id|"
+                    "schema_issuer_did|"
+                    "schema_name|"
+                    "schema_version|"
+                    "issuer_did|"
+                    "cred_def_id|"
+                    "attr::.+::value$"  # indy does not support attr::...::marker here
+                ),
+                example="cred_def_id",  # marshmallow/apispec v3.0 ignores
+            ),
+            values=fields.Str(example=INDY_CRED_DEF_ID["example"]),
+        ),
+        description=(
+            "If present, credential must satisfy one of given restrictions: specify "
+            "schema_id, schema_issuer_did, schema_name, schema_version, "
+            "issuer_did, cred_def_id, and/or attr::<attribute-name>::value "
+            "where <attribute-name> represents a credential attribute name"
+        ),
+        required=False,
+    )
+    non_revoked = fields.Nested(IndyProofReqNonRevokedSchema(), required=False)
+
+    @validates_schema
+    def validate_fields(self, data, **kwargs):
+        """
+        Validate schema fields.
+
+        Data must have exactly one of name or names; if names then restrictions are
+        mandatory.
+
+        Args:
+            data: The data to validate
+
+        Raises:
+            ValidationError: if data has both or neither of name and names
+
+        """
+        if ("name" in data) == ("names" in data):
+            raise ValidationError(
+                "Attribute specification must have either name or names but not both"
+            )
+        restrictions = data.get("restrictions")
+        if ("names" in data) and (not restrictions or all(not r for r in restrictions)):
+            raise ValidationError(
+                "Attribute specification on 'names' must have non-empty restrictions"
+            )
 
 
 class IndyProofReqPredSpecSchema(Schema):
@@ -133,15 +252,15 @@ class IndyProofReqPredSpecSchema(Schema):
     p_type = fields.String(
         description="Predicate type ('<', '<=', '>=', or '>')",
         required=True,
-        **INDY_PREDICATE
+        **INDY_PREDICATE,
     )
     p_value = fields.Integer(description="Threshold value", required=True)
     restrictions = fields.List(
-        fields.Nested(IndyProofReqSpecRestrictionsSchema()),
+        fields.Nested(IndyProofReqPredSpecRestrictionsSchema()),
         description="If present, credential must satisfy one of given restrictions",
         required=False,
     )
-    non_revoked = fields.Nested(IndyProofReqNonRevoked(), required=False)
+    non_revoked = fields.Nested(IndyProofReqNonRevokedSchema(), required=False)
 
 
 class IndyProofRequestSchema(Schema):
@@ -158,7 +277,7 @@ class IndyProofRequestSchema(Schema):
         description="Proof request version",
         required=False,
         default="1.0",
-        **INDY_VERSION
+        **INDY_VERSION,
     )
     requested_attributes = fields.Dict(
         description=("Requested attribute specifications of proof request"),
@@ -172,16 +291,29 @@ class IndyProofRequestSchema(Schema):
         keys=fields.Str(example="0_age_GE_uuid"),  # marshmallow/apispec v3.0 ignores
         values=fields.Nested(IndyProofReqPredSpecSchema()),
     )
+    non_revoked = fields.Nested(IndyProofReqNonRevokedSchema(), required=False)
 
 
-class V10PresentationRequestRequestSchema(AdminAPIMessageTracingSchema):
-    """Request schema for sending a proof request."""
+class V10PresentationCreateRequestRequestSchema(AdminAPIMessageTracingSchema):
+    """Request schema for creating a proof request free of any connection."""
+
+    proof_request = fields.Nested(IndyProofRequestSchema(), required=True)
+    comment = fields.Str(required=False)
+    trace = fields.Bool(
+        description="Whether to trace event (default false)",
+        required=False,
+        example=False,
+    )
+
+
+class V10PresentationSendRequestRequestSchema(
+    V10PresentationCreateRequestRequestSchema
+):
+    """Request schema for sending a proof request on a connection."""
 
     connection_id = fields.UUID(
         description="Connection identifier", required=True, example=UUIDFour.EXAMPLE
     )
-    proof_request = fields.Nested(IndyProofRequestSchema(), required=True)
-    comment = fields.Str(required=False)
 
 
 class IndyRequestedCredsRequestedAttrSchema(Schema):
@@ -192,9 +324,15 @@ class IndyRequestedCredsRequestedAttrSchema(Schema):
         description=(
             "Wallet credential identifier (typically but not necessarily a UUID)"
         ),
+        required=True,
     )
     revealed = fields.Bool(
-        description="Whether to reveal attribute in proof", default=True
+        description="Whether to reveal attribute in proof", required=True
+    )
+    timestamp = fields.Int(
+        description="Epoch timestamp of interest for non-revocation proof",
+        required=False,
+        **INT_EPOCH,
     )
 
 
@@ -202,10 +340,16 @@ class IndyRequestedCredsRequestedPredSchema(Schema):
     """Schema for requested predicates within indy requested credentials structure."""
 
     cred_id = fields.Str(
-        example="3fa85f64-5717-4562-b3fc-2c963f66afa6",
         description=(
             "Wallet credential identifier (typically but not necessarily a UUID)"
         ),
+        example="3fa85f64-5717-4562-b3fc-2c963f66afa6",
+        required=True,
+    )
+    timestamp = fields.Int(
+        description="Epoch timestamp of interest for non-revocation proof",
+        required=False,
+        **INT_EPOCH,
     )
 
 
@@ -242,9 +386,42 @@ class V10PresentationRequestSchema(AdminAPIMessageTracingSchema):
         keys=fields.Str(example="pred_referent"),  # marshmallow/apispec v3.0 ignores
         values=fields.Nested(IndyRequestedCredsRequestedPredSchema()),
     )
+    trace = fields.Bool(
+        description="Whether to trace event (default false)",
+        required=False,
+        example=False,
+    )
+
+
+class CredentialsFetchQueryStringSchema(Schema):
+    """Parameters and validators for credentials fetch request query string."""
+
+    referent = fields.Str(
+        description="Proof request referents of interest, comma-separated",
+        required=False,
+        example="1_name_uuid,2_score_uuid",
+    )
+    start = fields.Int(description="Start index", required=False, **WHOLE_NUM)
+    count = fields.Int(
+        description="Maximum number to retrieve", required=False, **NATURAL_NUM
+    )
+    extra_query = fields.Str(
+        description="(JSON) object mapping referents to extra WQL queries",
+        required=False,
+        **INDY_EXTRA_WQL,
+    )
+
+
+class PresExIdMatchInfoSchema(Schema):
+    """Path parameters and validators for request taking presentation exchange id."""
+
+    pres_ex_id = fields.Str(
+        description="Presentation exchange identifier", required=True, **UUID4
+    )
 
 
 @docs(tags=["present-proof"], summary="Fetch all present-proof exchange records")
+@querystring_schema(V10PresentationExchangeListQueryStringSchema)
 @response_schema(V10PresentationExchangeListSchema(), 200)
 async def presentation_exchange_list(request: web.BaseRequest):
     """
@@ -261,15 +438,23 @@ async def presentation_exchange_list(request: web.BaseRequest):
     tag_filter = {}
     if "thread_id" in request.query and request.query["thread_id"] != "":
         tag_filter["thread_id"] = request.query["thread_id"]
-    post_filter = {}
-    for param_name in ("connection_id", "role", "state"):
-        if param_name in request.query and request.query[param_name] != "":
-            post_filter[param_name] = request.query[param_name]
-    records = await V10PresentationExchange.query(context, tag_filter, post_filter)
-    return web.json_response({"results": [record.serialize() for record in records]})
+    post_filter = {
+        k: request.query[k]
+        for k in ("connection_id", "role", "state")
+        if request.query.get(k, "") != ""
+    }
+
+    try:
+        records = await V10PresentationExchange.query(context, tag_filter, post_filter)
+        results = [record.serialize() for record in records]
+    except (StorageError, BaseModelError) as err:
+        raise web.HTTPBadRequest(reason=err.roll_up) from err
+
+    return web.json_response({"results": results})
 
 
 @docs(tags=["present-proof"], summary="Fetch a single presentation exchange record")
+@match_info_schema(PresExIdMatchInfoSchema())
 @response_schema(V10PresentationExchangeSchema(), 200)
 async def presentation_exchange_retrieve(request: web.BaseRequest):
     """
@@ -283,40 +468,29 @@ async def presentation_exchange_retrieve(request: web.BaseRequest):
 
     """
     context = request.app["request_context"]
+    outbound_handler = request.app["outbound_message_router"]
+
     presentation_exchange_id = request.match_info["pres_ex_id"]
+    pres_ex_record = None
     try:
-        record = await V10PresentationExchange.retrieve_by_id(
+        pres_ex_record = await V10PresentationExchange.retrieve_by_id(
             context, presentation_exchange_id
         )
-    except StorageNotFoundError:
-        raise web.HTTPNotFound()
-    return web.json_response(record.serialize())
+        result = pres_ex_record.serialize()
+    except StorageNotFoundError as err:
+        raise web.HTTPNotFound(reason=err.roll_up) from err
+    except (BaseModelError, StorageError) as err:
+        await internal_error(err, web.HTTPBadRequest, pres_ex_record, outbound_handler)
+
+    return web.json_response(result)
 
 
 @docs(
     tags=["present-proof"],
     summary="Fetch credentials for a presentation request from wallet",
-    parameters=[
-        {
-            "name": "start",
-            "in": "query",
-            "schema": {"type": "string"},
-            "required": False,
-        },
-        {
-            "name": "count",
-            "in": "query",
-            "schema": {"type": "string"},
-            "required": False,
-        },
-        {
-            "name": "extra_query",
-            "in": "query",
-            "schema": {"type": "string"},
-            "required": False,
-        },
-    ],
 )
+@match_info_schema(PresExIdMatchInfoSchema())
+@querystring_schema(CredentialsFetchQueryStringSchema())
 async def presentation_exchange_credentials_list(request: web.BaseRequest):
     """
     Request handler for searching applicable credential records.
@@ -329,17 +503,20 @@ async def presentation_exchange_credentials_list(request: web.BaseRequest):
 
     """
     context = request.app["request_context"]
+    outbound_handler = request.app["outbound_message_router"]
 
     presentation_exchange_id = request.match_info["pres_ex_id"]
-    referents = request.match_info.get("referent")
-    presentation_referents = referents.split(",") if referents else ()
+    referents = request.query.get("referent")
+    presentation_referents = (
+        (r.strip() for r in referents.split(",")) if referents else ()
+    )
 
     try:
-        presentation_exchange_record = await V10PresentationExchange.retrieve_by_id(
+        pres_ex_record = await V10PresentationExchange.retrieve_by_id(
             context, presentation_exchange_id
         )
-    except StorageNotFoundError:
-        raise web.HTTPNotFound()
+    except StorageNotFoundError as err:
+        raise web.HTTPNotFound(reason=err.roll_up) from err
 
     start = request.query.get("start")
     count = request.query.get("count")
@@ -353,15 +530,18 @@ async def presentation_exchange_credentials_list(request: web.BaseRequest):
     count = int(count) if isinstance(count, str) else 10
 
     holder: BaseHolder = await context.inject(BaseHolder)
-    credentials = await holder.get_credentials_for_presentation_request_by_referent(
-        presentation_exchange_record.presentation_request,
-        presentation_referents,
-        start,
-        count,
-        extra_query,
-    )
+    try:
+        credentials = await holder.get_credentials_for_presentation_request_by_referent(
+            pres_ex_record.presentation_request,
+            presentation_referents,
+            start,
+            count,
+            extra_query,
+        )
+    except HolderError as err:
+        await internal_error(err, web.HTTPBadRequest, pres_ex_record, outbound_handler)
 
-    presentation_exchange_record.log_state(
+    pres_ex_record.log_state(
         context,
         "Retrieved presentation credentials",
         {
@@ -395,24 +575,28 @@ async def presentation_exchange_send_proposal(request: web.BaseRequest):
 
     body = await request.json()
 
+    comment = body.get("comment")
     connection_id = body.get("connection_id")
+
+    # Aries RFC 37 calls it a proposal in the proposal struct but it's of type preview
+    presentation_preview = body.get("presentation_proposal")
+    connection_record = None
     try:
         connection_record = await ConnectionRecord.retrieve_by_id(
             context, connection_id
         )
-    except StorageNotFoundError:
-        raise web.HTTPBadRequest()
+        presentation_proposal_message = PresentationProposal(
+            comment=comment,
+            presentation_proposal=PresentationPreview.deserialize(presentation_preview),
+        )
+    except (BaseModelError, StorageError) as err:
+        await internal_error(
+            err, web.HTTPBadRequest, connection_record, outbound_handler
+        )
 
     if not connection_record.is_ready:
-        raise web.HTTPForbidden()
+        raise web.HTTPForbidden(reason=f"Connection {connection_id} not ready")
 
-    comment = body.get("comment")
-    # Aries#0037 calls it a proposal in the proposal struct but it's of type preview
-    presentation_preview = body.get("presentation_proposal")
-    presentation_proposal_message = PresentationProposal(
-        comment=comment,
-        presentation_proposal=PresentationPreview.deserialize(presentation_preview),
-    )
     trace_msg = body.get("trace")
     presentation_proposal_message.assign_trace_decorator(
         context.settings, trace_msg,
@@ -422,14 +606,21 @@ async def presentation_exchange_send_proposal(request: web.BaseRequest):
     )
 
     presentation_manager = PresentationManager(context)
-
-    (
-        presentation_exchange_record
-    ) = await presentation_manager.create_exchange_for_proposal(
-        connection_id=connection_id,
-        presentation_proposal_message=presentation_proposal_message,
-        auto_present=auto_present,
-    )
+    pres_ex_record = None
+    try:
+        pres_ex_record = await presentation_manager.create_exchange_for_proposal(
+            connection_id=connection_id,
+            presentation_proposal_message=presentation_proposal_message,
+            auto_present=auto_present,
+        )
+        result = pres_ex_record.serialize()
+    except (BaseModelError, StorageError) as err:
+        await internal_error(
+            err,
+            web.HTTPBadRequest,
+            pres_ex_record or connection_record,
+            outbound_handler,
+        )
 
     await outbound_handler(presentation_proposal_message, connection_id=connection_id)
 
@@ -440,7 +631,7 @@ async def presentation_exchange_send_proposal(request: web.BaseRequest):
         perf_counter=r_time,
     )
 
-    return web.json_response(presentation_exchange_record.serialize())
+    return web.json_response(result)
 
 
 @docs(
@@ -449,7 +640,7 @@ async def presentation_exchange_send_proposal(request: web.BaseRequest):
     Creates a presentation request not bound to any proposal or existing connection
     """,
 )
-@request_schema(V10PresentationRequestRequestSchema())
+@request_schema(V10PresentationCreateRequestRequestSchema())
 @response_schema(V10PresentationExchangeSchema(), 200)
 async def presentation_exchange_create_request(request: web.BaseRequest):
     """
@@ -492,12 +683,15 @@ async def presentation_exchange_create_request(request: web.BaseRequest):
     )
 
     presentation_manager = PresentationManager(context)
-
-    (
-        presentation_exchange_record
-    ) = await presentation_manager.create_exchange_for_request(
-        connection_id=None, presentation_request_message=presentation_request_message
-    )
+    pres_ex_record = None
+    try:
+        (pres_ex_record) = await presentation_manager.create_exchange_for_request(
+            connection_id=None,
+            presentation_request_message=presentation_request_message,
+        )
+        result = pres_ex_record.serialize()
+    except (BaseModelError, StorageError) as err:
+        await internal_error(err, web.HTTPBadRequest, pres_ex_record, outbound_handler)
 
     await outbound_handler(presentation_request_message, connection_id=None)
 
@@ -508,14 +702,14 @@ async def presentation_exchange_create_request(request: web.BaseRequest):
         perf_counter=r_time,
     )
 
-    return web.json_response(presentation_exchange_record.serialize())
+    return web.json_response(result)
 
 
 @docs(
     tags=["present-proof"],
     summary="Sends a free presentation request not bound to any proposal",
 )
-@request_schema(V10PresentationRequestRequestSchema())
+@request_schema(V10PresentationSendRequestRequestSchema())
 @response_schema(V10PresentationExchangeSchema(), 200)
 async def presentation_exchange_send_free_request(request: web.BaseRequest):
     """
@@ -540,11 +734,11 @@ async def presentation_exchange_send_free_request(request: web.BaseRequest):
         connection_record = await ConnectionRecord.retrieve_by_id(
             context, connection_id
         )
-    except StorageNotFoundError:
-        raise web.HTTPBadRequest()
+    except StorageNotFoundError as err:
+        raise web.HTTPBadRequest(reason=err.roll_up) from err
 
     if not connection_record.is_ready:
-        raise web.HTTPForbidden()
+        raise web.HTTPForbidden(reason=f"Connection {connection_id} not ready")
 
     comment = body.get("comment")
     indy_proof_request = body.get("proof_request")
@@ -566,13 +760,20 @@ async def presentation_exchange_send_free_request(request: web.BaseRequest):
     )
 
     presentation_manager = PresentationManager(context)
-
-    (
-        presentation_exchange_record
-    ) = await presentation_manager.create_exchange_for_request(
-        connection_id=connection_id,
-        presentation_request_message=presentation_request_message,
-    )
+    pres_ex_record = None
+    try:
+        (pres_ex_record) = await presentation_manager.create_exchange_for_request(
+            connection_id=connection_id,
+            presentation_request_message=presentation_request_message,
+        )
+        result = pres_ex_record.serialize()
+    except (BaseModelError, StorageError) as err:
+        await internal_error(
+            err,
+            web.HTTPBadRequest,
+            pres_ex_record or connection_record,
+            outbound_handler,
+        )
 
     await outbound_handler(presentation_request_message, connection_id=connection_id)
 
@@ -583,14 +784,15 @@ async def presentation_exchange_send_free_request(request: web.BaseRequest):
         perf_counter=r_time,
     )
 
-    return web.json_response(presentation_exchange_record.serialize())
+    return web.json_response(result)
 
 
 @docs(
     tags=["present-proof"],
     summary="Sends a presentation request in reference to a proposal",
 )
-@request_schema(V10PresentationRequestRequestSchema())
+@match_info_schema(PresExIdMatchInfoSchema())
+@request_schema(V10PresentationSendRequestRequestSchema())
 @response_schema(V10PresentationExchangeSchema(), 200)
 async def presentation_exchange_send_bound_request(request: web.BaseRequest):
     """
@@ -609,12 +811,17 @@ async def presentation_exchange_send_bound_request(request: web.BaseRequest):
     outbound_handler = request.app["outbound_message_router"]
 
     presentation_exchange_id = request.match_info["pres_ex_id"]
-    presentation_exchange_record = await V10PresentationExchange.retrieve_by_id(
+    pres_ex_record = await V10PresentationExchange.retrieve_by_id(
         context, presentation_exchange_id
     )
-    assert presentation_exchange_record.state == (
-        V10PresentationExchange.STATE_PROPOSAL_RECEIVED
-    )
+    if pres_ex_record.state != (V10PresentationExchange.STATE_PROPOSAL_RECEIVED):
+        raise web.HTTPBadRequest(
+            reason=(
+                f"Presentation exchange {presentation_exchange_id} "
+                f"in {pres_ex_record.state} state "
+                f"(must be {V10PresentationExchange.STATE_PROPOSAL_RECEIVED})"
+            )
+        )
     body = await request.json()
 
     connection_id = body.get("connection_id")
@@ -622,23 +829,31 @@ async def presentation_exchange_send_bound_request(request: web.BaseRequest):
         connection_record = await ConnectionRecord.retrieve_by_id(
             context, connection_id
         )
-    except StorageNotFoundError:
-        raise web.HTTPBadRequest()
+    except StorageError as err:
+        raise web.HTTPBadRequest(reason=err.roll_up) from err
 
     if not connection_record.is_ready:
-        raise web.HTTPForbidden()
+        raise web.HTTPForbidden(reason=f"Connection {connection_id} not ready")
 
     presentation_manager = PresentationManager(context)
+    try:
+        (
+            pres_ex_record,
+            presentation_request_message,
+        ) = await presentation_manager.create_bound_request(pres_ex_record)
+        result = pres_ex_record.serialize()
+    except (BaseModelError, StorageError) as err:
+        await internal_error(
+            err,
+            web.HTTPBadRequest,
+            pres_ex_record or connection_record,
+            outbound_handler,
+        )
 
-    (
-        presentation_exchange_record,
-        presentation_request_message,
-    ) = await presentation_manager.create_bound_request(presentation_exchange_record)
     trace_msg = body.get("trace")
     presentation_request_message.assign_trace_decorator(
         context.settings, trace_msg,
     )
-
     await outbound_handler(presentation_request_message, connection_id=connection_id)
 
     trace_event(
@@ -648,10 +863,11 @@ async def presentation_exchange_send_bound_request(request: web.BaseRequest):
         perf_counter=r_time,
     )
 
-    return web.json_response(presentation_exchange_record.serialize())
+    return web.json_response(result)
 
 
 @docs(tags=["present-proof"], summary="Sends a proof presentation")
+@match_info_schema(PresExIdMatchInfoSchema())
 @request_schema(V10PresentationRequestSchema())
 @response_schema(V10PresentationExchangeSchema())
 async def presentation_exchange_send_presentation(request: web.BaseRequest):
@@ -670,46 +886,58 @@ async def presentation_exchange_send_presentation(request: web.BaseRequest):
     context = request.app["request_context"]
     outbound_handler = request.app["outbound_message_router"]
     presentation_exchange_id = request.match_info["pres_ex_id"]
-    presentation_exchange_record = await V10PresentationExchange.retrieve_by_id(
+    pres_ex_record = await V10PresentationExchange.retrieve_by_id(
         context, presentation_exchange_id
     )
+    if pres_ex_record.state != (V10PresentationExchange.STATE_REQUEST_RECEIVED):
+        raise web.HTTPBadRequest(
+            reason=(
+                f"Presentation exchange {presentation_exchange_id} "
+                f"in {pres_ex_record.state} state "
+                f"(must be {V10PresentationExchange.STATE_REQUEST_RECEIVED})"
+            )
+        )
 
     body = await request.json()
 
-    connection_id = presentation_exchange_record.connection_id
+    connection_id = pres_ex_record.connection_id
     try:
         connection_record = await ConnectionRecord.retrieve_by_id(
             context, connection_id
         )
-    except StorageNotFoundError:
-        raise web.HTTPBadRequest()
+    except StorageNotFoundError as err:
+        raise web.HTTPBadRequest(reason=err.roll_up) from err
 
     if not connection_record.is_ready:
-        raise web.HTTPForbidden()
-
-    assert (
-        presentation_exchange_record.state
-    ) == V10PresentationExchange.STATE_REQUEST_RECEIVED
+        raise web.HTTPForbidden(reason=f"Connection {connection_id} not ready")
 
     presentation_manager = PresentationManager(context)
+    try:
+        (
+            pres_ex_record,
+            presentation_message,
+        ) = await presentation_manager.create_presentation(
+            pres_ex_record,
+            {
+                "self_attested_attributes": body.get("self_attested_attributes"),
+                "requested_attributes": body.get("requested_attributes"),
+                "requested_predicates": body.get("requested_predicates"),
+            },
+            comment=body.get("comment"),
+        )
+        result = pres_ex_record.serialize()
+    except (BaseModelError, HolderError, LedgerError, StorageError) as err:
+        await internal_error(
+            err,
+            web.HTTPBadRequest,
+            pres_ex_record or connection_record,
+            outbound_handler,
+        )
 
-    (
-        presentation_exchange_record,
-        presentation_message,
-    ) = await presentation_manager.create_presentation(
-        presentation_exchange_record,
-        {
-            "self_attested_attributes": body.get("self_attested_attributes"),
-            "requested_attributes": body.get("requested_attributes"),
-            "requested_predicates": body.get("requested_predicates"),
-        },
-        comment=body.get("comment"),
-    )
     trace_msg = body.get("trace")
     presentation_message.assign_trace_decorator(
         context.settings, trace_msg,
     )
-
     await outbound_handler(presentation_message, connection_id=connection_id)
 
     trace_event(
@@ -719,10 +947,11 @@ async def presentation_exchange_send_presentation(request: web.BaseRequest):
         perf_counter=r_time,
     )
 
-    return web.json_response(presentation_exchange_record.serialize())
+    return web.json_response(result)
 
 
 @docs(tags=["present-proof"], summary="Verify a received presentation")
+@match_info_schema(PresExIdMatchInfoSchema())
 @response_schema(V10PresentationExchangeSchema())
 async def presentation_exchange_verify_presentation(request: web.BaseRequest):
     """
@@ -738,44 +967,53 @@ async def presentation_exchange_verify_presentation(request: web.BaseRequest):
     r_time = get_timer()
 
     context = request.app["request_context"]
+    outbound_handler = request.app["outbound_message_router"]
+
     presentation_exchange_id = request.match_info["pres_ex_id"]
 
-    presentation_exchange_record = await V10PresentationExchange.retrieve_by_id(
+    pres_ex_record = await V10PresentationExchange.retrieve_by_id(
         context, presentation_exchange_id
     )
-    connection_id = presentation_exchange_record.connection_id
+    if pres_ex_record.state != (V10PresentationExchange.STATE_PRESENTATION_RECEIVED):
+        raise web.HTTPBadRequest(
+            reason=(
+                f"Presentation exchange {presentation_exchange_id} "
+                f"in {pres_ex_record.state} state "
+                f"(must be {V10PresentationExchange.STATE_PRESENTATION_RECEIVED})"
+            )
+        )
+
+    connection_id = pres_ex_record.connection_id
 
     try:
         connection_record = await ConnectionRecord.retrieve_by_id(
             context, connection_id
         )
-    except StorageNotFoundError:
-        raise web.HTTPBadRequest()
+    except StorageError as err:
+        raise web.HTTPBadRequest(reason=err.roll_up) from err
 
     if not connection_record.is_ready:
-        raise web.HTTPForbidden()
-
-    assert (
-        presentation_exchange_record.state
-    ) == V10PresentationExchange.STATE_PRESENTATION_RECEIVED
+        raise web.HTTPForbidden(reason=f"Connection {connection_id} not ready")
 
     presentation_manager = PresentationManager(context)
-
-    presentation_exchange_record = await presentation_manager.verify_presentation(
-        presentation_exchange_record
-    )
+    try:
+        pres_ex_record = await presentation_manager.verify_presentation(pres_ex_record)
+        result = pres_ex_record.serialize()
+    except (LedgerError, BaseModelError) as err:
+        await internal_error(err, web.HTTPBadRequest, pres_ex_record, outbound_handler)
 
     trace_event(
         context.settings,
-        presentation_exchange_record,
+        pres_ex_record,
         outcome="presentation_exchange_verify.END",
         perf_counter=r_time,
     )
 
-    return web.json_response(presentation_exchange_record.serialize())
+    return web.json_response(result)
 
 
 @docs(tags=["present-proof"], summary="Remove an existing presentation exchange record")
+@match_info_schema(PresExIdMatchInfoSchema())
 async def presentation_exchange_remove(request: web.BaseRequest):
     """
     Request handler for removing a presentation exchange record.
@@ -785,15 +1023,20 @@ async def presentation_exchange_remove(request: web.BaseRequest):
 
     """
     context = request.app["request_context"]
+    outbound_handler = request.app["outbound_message_router"]
+
     presentation_exchange_id = request.match_info["pres_ex_id"]
+    pres_ex_record = None
     try:
-        presentation_exchange_record = await V10PresentationExchange.retrieve_by_id(
+        pres_ex_record = await V10PresentationExchange.retrieve_by_id(
             context, presentation_exchange_id
         )
-    except StorageNotFoundError:
-        raise web.HTTPNotFound()
+        await pres_ex_record.delete_record(context)
+    except StorageNotFoundError as err:
+        await internal_error(err, web.HTTPNotFound, pres_ex_record, outbound_handler)
+    except StorageError as err:
+        await internal_error(err, web.HTTPBadRequest, pres_ex_record, outbound_handler)
 
-    await presentation_exchange_record.delete_record(context)
     return web.json_response({})
 
 
@@ -802,26 +1045,27 @@ async def register(app: web.Application):
 
     app.add_routes(
         [
-            web.get("/present-proof/records", presentation_exchange_list),
             web.get(
-                "/present-proof/records/{pres_ex_id}", presentation_exchange_retrieve
+                "/present-proof/records", presentation_exchange_list, allow_head=False
+            ),
+            web.get(
+                "/present-proof/records/{pres_ex_id}",
+                presentation_exchange_retrieve,
+                allow_head=False,
             ),
             web.get(
                 "/present-proof/records/{pres_ex_id}/credentials",
                 presentation_exchange_credentials_list,
-            ),
-            web.get(
-                "/present-proof/records/{pres_ex_id}/credentials/{referent}",
-                presentation_exchange_credentials_list,
+                allow_head=False,
             ),
             web.post(
-                "/present-proof/send-proposal", presentation_exchange_send_proposal
+                "/present-proof/send-proposal", presentation_exchange_send_proposal,
             ),
             web.post(
-                "/present-proof/create-request", presentation_exchange_create_request
+                "/present-proof/create-request", presentation_exchange_create_request,
             ),
             web.post(
-                "/present-proof/send-request", presentation_exchange_send_free_request
+                "/present-proof/send-request", presentation_exchange_send_free_request,
             ),
             web.post(
                 "/present-proof/records/{pres_ex_id}/send-request",
@@ -841,3 +1085,18 @@ async def register(app: web.Application):
             ),
         ]
     )
+
+
+def post_process_routes(app: web.Application):
+    """Amend swagger API."""
+
+    # Add top-level tags description
+    if "tags" not in app._state["swagger_dict"]:
+        app._state["swagger_dict"]["tags"] = []
+    app._state["swagger_dict"]["tags"].append(
+        {
+            "name": "present-proof",
+            "description": "Proof presentation",
+            "externalDocs": {"description": "Specification", "url": SPEC_URI},
+        }
+    )
