@@ -111,13 +111,21 @@ class TestPresentationManager(AsyncTestCase):
 
         Holder = async_mock.MagicMock(IndyHolder, autospec=True)
         self.holder = Holder()
-        self.holder.get_credentials_for_presentation_request_by_referent = async_mock.CoroutineMock(
+        get_creds = async_mock.CoroutineMock(
             return_value=(
                 {
-                    "cred_info": {"referent": "dummy_reft"}
+                    "cred_info": {
+                        "referent": "dummy_reft",
+                        "attrs": {
+                            "player": "Richie Knucklez",
+                            "screenCapture": "aW1hZ2luZSBhIHNjcmVlbiBjYXB0dXJl",
+                            "highScore": "1234560",
+                        },
+                    }
                 },  # leave this comma: return a tuple
             )
         )
+        self.holder.get_credentials_for_presentation_request_by_referent = get_creds
         self.holder.get_credential = async_mock.CoroutineMock(
             return_value=json.dumps(
                 {
@@ -314,6 +322,76 @@ class TestPresentationManager(AsyncTestCase):
             req_creds = await indy_proof_req_preview2indy_requested_creds(
                 indy_proof_req, holder=self.holder
             )
+            assert not req_creds["self_attested_attributes"]
+            assert len(req_creds["requested_attributes"]) == 2
+            assert len(req_creds["requested_predicates"]) == 1
+
+            (exchange_out, pres_msg) = await self.manager.create_presentation(
+                exchange_in, req_creds
+            )
+            save_ex.assert_called_once()
+            assert exchange_out.state == V10PresentationExchange.STATE_PRESENTATION_SENT
+
+    async def test_create_presentation_self_asserted(self):
+        self.context.connection_record = async_mock.MagicMock()
+        self.context.connection_record.connection_id = CONN_ID
+        PRES_PREVIEW_SELFIE = PresentationPreview(
+            attributes=[
+                PresAttrSpec(name="player", value="Richie Knucklez"),
+                PresAttrSpec(
+                    name="screenCapture",
+                    mime_type="image/png",
+                    value="aW1hZ2luZSBhIHNjcmVlbiBjYXB0dXJl",
+                ),
+            ],
+            predicates=[
+                PresPredSpec(
+                    name="highScore",
+                    cred_def_id=None,
+                    predicate=">=",
+                    threshold=1000000,
+                )
+            ],
+        )
+
+        exchange_in = V10PresentationExchange()
+        indy_proof_req = await PRES_PREVIEW_SELFIE.indy_proof_request(
+            name=PROOF_REQ_NAME,
+            version=PROOF_REQ_VERSION,
+            nonce=PROOF_REQ_NONCE,
+            ledger=await self.context.inject(BaseLedger, required=False),
+        )
+
+        exchange_in.presentation_request = indy_proof_req
+        request = async_mock.MagicMock()
+        request.indy_proof_request = async_mock.MagicMock()
+        request._thread_id = "dummy"
+        self.context.message = request
+
+        more_magic_rr = async_mock.MagicMock(
+            get_or_fetch_local_tails_path=async_mock.CoroutineMock(
+                return_value="/tmp/sample/tails/path"
+            )
+        )
+        with async_mock.patch.object(
+            V10PresentationExchange, "save", autospec=True
+        ) as save_ex, async_mock.patch.object(
+            test_module, "AttachDecorator", autospec=True
+        ) as mock_attach_decorator, async_mock.patch.object(
+            test_module, "RevocationRegistry", autospec=True
+        ) as mock_rr:
+            mock_rr.from_definition = async_mock.MagicMock(return_value=more_magic_rr)
+
+            mock_attach_decorator.from_indy_dict = async_mock.MagicMock(
+                return_value=mock_attach_decorator
+            )
+
+            req_creds = await indy_proof_req_preview2indy_requested_creds(
+                indy_proof_req, holder=self.holder
+            )
+            assert len(req_creds["self_attested_attributes"]) == 3
+            assert not req_creds["requested_attributes"]
+            assert not req_creds["requested_predicates"]
 
             (exchange_out, pres_msg) = await self.manager.create_presentation(
                 exchange_in, req_creds
@@ -341,13 +419,19 @@ class TestPresentationManager(AsyncTestCase):
 
         Holder = async_mock.MagicMock(IndyHolder, autospec=True)
         self.holder = Holder()
-        self.holder.get_credentials_for_presentation_request_by_referent = async_mock.CoroutineMock(
+        get_creds = async_mock.CoroutineMock(
             return_value=(
                 {
-                    "cred_info": {"referent": "dummy_reft"}
+                    "cred_info": {"referent": "dummy_reft"},
+                    "attrs": {
+                        "player": "Richie Knucklez",
+                        "screenCapture": "aW1hZ2luZSBhIHNjcmVlbiBjYXB0dXJl",
+                        "highScore": "1234560",
+                    },
                 },  # leave this comma: return a tuple
             )
         )
+        self.holder.get_credentials_for_presentation_request_by_referent = get_creds
         self.holder.get_credential = async_mock.CoroutineMock(
             return_value=json.dumps(
                 {
@@ -419,13 +503,20 @@ class TestPresentationManager(AsyncTestCase):
 
         Holder = async_mock.MagicMock(IndyHolder, autospec=True)
         self.holder = Holder()
-        self.holder.get_credentials_for_presentation_request_by_referent = async_mock.CoroutineMock(
+        get_creds = async_mock.CoroutineMock(
             return_value=(
                 {
-                    "cred_info": {"referent": "dummy_reft"}
+                    "cred_info": {"referent": "dummy_reft"},
+                    "attrs": {
+                        "player": "Richie Knucklez",
+                        "screenCapture": "aW1hZ2luZSBhIHNjcmVlbiBjYXB0dXJl",
+                        "highScore": "1234560",
+                    },
                 },  # leave this comma: return a tuple
             )
         )
+        self.holder.get_credentials_for_presentation_request_by_referent = get_creds
+
         self.holder.get_credential = async_mock.CoroutineMock(
             return_value=json.dumps(
                 {
@@ -438,7 +529,7 @@ class TestPresentationManager(AsyncTestCase):
         )
         self.holder.create_presentation = async_mock.CoroutineMock(return_value="{}")
         self.holder.create_revocation_state = async_mock.CoroutineMock(
-            side_effect=test_module.HolderError(706, {"message": "Nope"})
+            side_effect=test_module.HolderError("Problem", {"message": "Nope"})
         )
         self.context.injector.clear_binding(BaseHolder)
         self.context.injector.bind_instance(BaseHolder, self.holder)
@@ -476,18 +567,27 @@ class TestPresentationManager(AsyncTestCase):
             nonce=PROOF_REQ_NONCE,
             ledger=await self.context.inject(BaseLedger, required=False),
         )
-        self.holder.get_credentials_for_presentation_request_by_referent.return_value = ()
+        get_creds = async_mock.CoroutineMock(return_value=())
+        self.holder.get_credentials_for_presentation_request_by_referent = get_creds
 
         with self.assertRaises(ValueError):
             await indy_proof_req_preview2indy_requested_creds(
                 indy_proof_req, holder=self.holder
             )
 
-        self.holder.get_credentials_for_presentation_request_by_referent.return_value = (
-            {
-                "cred_info": {"referent": "dummy_reft"}
-            },  # leave this comma: return a tuple
+        get_creds = async_mock.CoroutineMock(
+            return_value=(
+                {
+                    "cred_info": {"referent": "dummy_reft"},
+                    "attrs": {
+                        "player": "Richie Knucklez",
+                        "screenCapture": "aW1hZ2luZSBhIHNjcmVlbiBjYXB0dXJl",
+                        "highScore": "1234560",
+                    },
+                },  # leave this comma: return a tuple
+            )
         )
+        self.holder.get_credentials_for_presentation_request_by_referent = get_creds
 
     async def test_receive_presentation(self):
         self.context.connection_record = async_mock.MagicMock()
